@@ -2,6 +2,7 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include "../configure.h"
 #include "../include/aftersteplib.h"
 
 /***************************************************************************
@@ -12,7 +13,7 @@
 void
 CopyString (char **dest, char *source)
 {
-  int len;
+  size_t len;
   char *start;
 
   while (((isspace ((unsigned char) *source)) && (*source != '\n')) && (*source != 0))
