@@ -9,7 +9,7 @@
 #include <sys/utsname.h>
 
 int
-mygethostname (char *client, int length)
+mygethostname (char *client, size_t length)
 {
   struct utsname sysname;
 
@@ -23,14 +23,14 @@ mygethostname (char *client, int length)
 /* define mygethostname() by using gethostname() :-) */
 
 int
-mygethostname (char *client, int length)
+mygethostname (char *client, size_t length)
 {
   gethostname (client, length);
 }
 
 #else
 int
-mygethostname (char *client, int length)
+mygethostname (char *client, size_t length)
 {
   *client = 0;
 }
