@@ -18,19 +18,29 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <signal.h>
-#include <time.h>
-#include <limits.h>
-#include <math.h>
-#include <unistd.h>
-#include <string.h>
 #include <ctype.h>
+#include <errno.h>
+#include <math.h>
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <time.h>
+#include <string.h>
+
+#include <limits.h>
+#include <unistd.h>
+
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
+
+#define MODULE_X_INTERFACE
+
 #include "../../configure.h"
-#include "../../include/module.h"
 #include "../../include/aftersteplib.h"
+#include "../../include/afterstep.h"
+#include "../../include/style.h"
+#include "../../include/screen.h"
+#include "../../include/module.h"
 #include "Animate.h"
 
 #define AS_PI 3.14159265358979323846
@@ -39,10 +49,22 @@ Display *dpy;
 int screen;
 char *MyName = NULL;
 
-Window root;
+ScreenInfo Scr;			/* root window */
 GC DrawGC;
-char *ProgName;
-int Channel[2];
+char *MyName;
+
+/* File type information */
+static int fd_width;
+static int fd[2];
+static int x_fd;
+
+/* masks for AS pipe */
+#define mask_reg (	M_CONFIGURE_WINDOW |	\
+			M_ICONIFY |		\
+			M_DEICONIFY |		\
+			M_LOCKONSEND	)
+#define mask_off 0
+
 void Loop ();
 void ParseOptions (const char *);
 
@@ -96,10 +118,10 @@ AnimateResizeTwist (int x, int y, int w, int h, int fx, int fy, int fw, int fh)
       points[4].x = cx + cos (angle - a) * d;
       points[4].y = cy + sin (angle - a) * d;
       XGrabServer (dpy);
-      XDrawLines (dpy, root, DrawGC, points, 5, CoordModeOrigin);
+      XDrawLines (dpy, Scr.Root, DrawGC, points, 5, CoordModeOrigin);
       XFlush (dpy);
       sleep_a_little (Animate.delay * 1000);
-      XDrawLines (dpy, root, DrawGC, points, 5, CoordModeOrigin);
+      XDrawLines (dpy, Scr.Root, DrawGC, points, 5, CoordModeOrigin);
       XUngrabServer (dpy);
       cx += xstep;
       cy += ystep;
@@ -166,10 +188,10 @@ AnimateResizeFlip (int x, int y, int w, int h, int fx, int fy, int fw, int fh)
       points[4].y = points[0].y;
 
       XGrabServer (dpy);
-      XDrawLines (dpy, root, DrawGC, points, 5, CoordModeOrigin);
+      XDrawLines (dpy, Scr.Root, DrawGC, points, 5, CoordModeOrigin);
       XFlush (dpy);
       sleep_a_little (Animate.delay * 1000);
-      XDrawLines (dpy, root, DrawGC, points, 5, CoordModeOrigin);
+      XDrawLines (dpy, Scr.Root, DrawGC, points, 5, CoordModeOrigin);
       XUngrabServer (dpy);
       cx += xstep;
       cy += ystep;
@@ -230,10 +252,10 @@ AnimateResizeTurn (int x, int y, int w, int h, int fx, int fy, int fw, int fh)
       points[4].y = points[0].y;
 
       XGrabServer (dpy);
-      XDrawLines (dpy, root, DrawGC, points, 5, CoordModeOrigin);
+      XDrawLines (dpy, Scr.Root, DrawGC, points, 5, CoordModeOrigin);
       XFlush (dpy);
       sleep_a_little (Animate.delay * 1000);
-      XDrawLines (dpy, root, DrawGC, points, 5, CoordModeOrigin);
+      XDrawLines (dpy, Scr.Root, DrawGC, points, 5, CoordModeOrigin);
       XUngrabServer (dpy);
       cx += xstep;
       cy += ystep;
@@ -270,10 +292,10 @@ AnimateResizeZoom (int x, int y, int w, int h, int fx, int fy, int fw, int fh)
   for (i = 0; i < Animate.iterations; i++)
     {
       XGrabServer (dpy);
-      XDrawRectangle (dpy, root, DrawGC, (int) cx, (int) cy, (int) cw, (int) ch);
+      XDrawRectangle (dpy, Scr.Root, DrawGC, (int) cx, (int) cy, (int) cw, (int) ch);
       XFlush (dpy);
       sleep_a_little (Animate.delay);
-      XDrawRectangle (dpy, root, DrawGC, (int) cx, (int) cy, (int) cw, (int) ch);
+      XDrawRectangle (dpy, Scr.Root, DrawGC, (int) cx, (int) cy, (int) cw, (int) ch);
       XUngrabServer (dpy);
       cx += xstep;
       cy += ystep;
@@ -316,29 +338,29 @@ AnimateResizeZoom3D (int x, int y, int w, int h, int fx, int fy, int fw, int fh)
       for (i = 0; i < Animate.iterations; i++)
 	{
 	  XGrabServer (dpy);
-	  XDrawRectangle (dpy, root, DrawGC, (int) cx, (int) cy, (int) cw,
+	  XDrawRectangle (dpy, Scr.Root, DrawGC, (int) cx, (int) cy, (int) cw,
 			  (int) ch);
-	  XDrawRectangle (dpy, root, DrawGC, (int) fx, (int) fy, (int) fw,
+	  XDrawRectangle (dpy, Scr.Root, DrawGC, (int) fx, (int) fy, (int) fw,
 			  (int) fh);
-	  XDrawLine (dpy, root, DrawGC, (int) cx, (int) cy, fx, fy);
-	  XDrawLine (dpy, root, DrawGC, ((int) cx + (int) cw), (int) cy,
+	  XDrawLine (dpy, Scr.Root, DrawGC, (int) cx, (int) cy, fx, fy);
+	  XDrawLine (dpy, Scr.Root, DrawGC, ((int) cx + (int) cw), (int) cy,
 		     (fx + fw), fy);
-	  XDrawLine (dpy, root, DrawGC, ((int) cx + (int) cw),
+	  XDrawLine (dpy, Scr.Root, DrawGC, ((int) cx + (int) cw),
 		     ((int) cy + (int) ch), (fx + fw), (fy + fh));
-	  XDrawLine (dpy, root, DrawGC, (int) cx, ((int) cy + (int) ch), fx,
+	  XDrawLine (dpy, Scr.Root, DrawGC, (int) cx, ((int) cy + (int) ch), fx,
 		     (fy + fh));
 	  XFlush (dpy);
 	  sleep_a_little (Animate.delay);
-	  XDrawRectangle (dpy, root, DrawGC, (int) cx, (int) cy, (int) cw,
+	  XDrawRectangle (dpy, Scr.Root, DrawGC, (int) cx, (int) cy, (int) cw,
 			  (int) ch);
-	  XDrawRectangle (dpy, root, DrawGC, (int) fx, (int) fy, (int) fw,
+	  XDrawRectangle (dpy, Scr.Root, DrawGC, (int) fx, (int) fy, (int) fw,
 			  (int) fh);
-	  XDrawLine (dpy, root, DrawGC, (int) cx, (int) cy, fx, fy);
-	  XDrawLine (dpy, root, DrawGC, ((int) cx + (int) cw), (int) cy,
+	  XDrawLine (dpy, Scr.Root, DrawGC, (int) cx, (int) cy, fx, fy);
+	  XDrawLine (dpy, Scr.Root, DrawGC, ((int) cx + (int) cw), (int) cy,
 		     (fx + fw), fy);
-	  XDrawLine (dpy, root, DrawGC, ((int) cx + (int) cw),
+	  XDrawLine (dpy, Scr.Root, DrawGC, ((int) cx + (int) cw),
 		     ((int) cy + (int) ch), (fx + fw), (fy + fh));
-	  XDrawLine (dpy, root, DrawGC, (int) cx, ((int) cy + (int) ch), fx,
+	  XDrawLine (dpy, Scr.Root, DrawGC, (int) cx, ((int) cy + (int) ch), fx,
 		     (fy + fh));
 	  XUngrabServer (dpy);
 	  cx += xstep;
@@ -353,27 +375,27 @@ AnimateResizeZoom3D (int x, int y, int w, int h, int fx, int fy, int fw, int fh)
       for (i = 0; i < Animate.iterations; i++)
 	{
 	  XGrabServer (dpy);
-	  XDrawRectangle (dpy, root, DrawGC, (int) cx, (int) cy, (int) cw,
+	  XDrawRectangle (dpy, Scr.Root, DrawGC, (int) cx, (int) cy, (int) cw,
 			  (int) ch);
-	  XDrawRectangle (dpy, root, DrawGC, x, y, w, h);
-	  XDrawLine (dpy, root, DrawGC, (int) cx, (int) cy, x, y);
-	  XDrawLine (dpy, root, DrawGC, ((int) cx + (int) cw), (int) cy,
+	  XDrawRectangle (dpy, Scr.Root, DrawGC, x, y, w, h);
+	  XDrawLine (dpy, Scr.Root, DrawGC, (int) cx, (int) cy, x, y);
+	  XDrawLine (dpy, Scr.Root, DrawGC, ((int) cx + (int) cw), (int) cy,
 		     (x + w), y);
-	  XDrawLine (dpy, root, DrawGC, ((int) cx + (int) cw), ((int) cy +
+	  XDrawLine (dpy, Scr.Root, DrawGC, ((int) cx + (int) cw), ((int) cy +
 					       (int) ch), (x + w), (y + h));
-	  XDrawLine (dpy, root, DrawGC, (int) cx, ((int) cy + (int) ch), x,
+	  XDrawLine (dpy, Scr.Root, DrawGC, (int) cx, ((int) cy + (int) ch), x,
 		     (y + h));
 	  XFlush (dpy);
 	  sleep_a_little (Animate.delay);
-	  XDrawRectangle (dpy, root, DrawGC, (int) cx, (int) cy, (int) cw,
+	  XDrawRectangle (dpy, Scr.Root, DrawGC, (int) cx, (int) cy, (int) cw,
 			  (int) ch);
-	  XDrawRectangle (dpy, root, DrawGC, x, y, w, h);
-	  XDrawLine (dpy, root, DrawGC, (int) cx, (int) cy, x, y);
-	  XDrawLine (dpy, root, DrawGC, ((int) cx + (int) cw), (int) cy,
+	  XDrawRectangle (dpy, Scr.Root, DrawGC, x, y, w, h);
+	  XDrawLine (dpy, Scr.Root, DrawGC, (int) cx, (int) cy, x, y);
+	  XDrawLine (dpy, Scr.Root, DrawGC, ((int) cx + (int) cw), (int) cy,
 		     (x + w), y);
-	  XDrawLine (dpy, root, DrawGC, ((int) cx + (int) cw),
+	  XDrawLine (dpy, Scr.Root, DrawGC, ((int) cx + (int) cw),
 		     ((int) cy + (int) ch), (x + w), (y + h));
-	  XDrawLine (dpy, root, DrawGC, (int) cx, ((int) cy + (int) ch), x,
+	  XDrawLine (dpy, Scr.Root, DrawGC, (int) cx, ((int) cy + (int) ch), x,
 		     (y + h));
 	  XUngrabServer (dpy);
 	  cx += xstep;
@@ -431,10 +453,10 @@ AnimateClose (int x, int y, int w, int h)
 	}
       for (i = h; i >= 2; i -= step)
 	{
-	  XDrawRectangle (dpy, root, DrawGC, x, y, w, i);
+	  XDrawRectangle (dpy, Scr.Root, DrawGC, x, y, w, i);
 	  XFlush (dpy);
 	  sleep_a_little (ANIM_DELAY2 * 600);
-	  XDrawRectangle (dpy, root, DrawGC, x, y, w, i);
+	  XDrawRectangle (dpy, Scr.Root, DrawGC, x, y, w, i);
 	  y += step / 2;
 	}
     }
@@ -447,10 +469,10 @@ AnimateClose (int x, int y, int w, int h)
     }
   for (i = w; i >= 0; i -= step)
     {
-      XDrawRectangle (dpy, root, DrawGC, x, y, i, 2);
+      XDrawRectangle (dpy, Scr.Root, DrawGC, x, y, i, 2);
       XFlush (dpy);
       sleep_a_little (ANIM_DELAY2 * 1000);
-      XDrawRectangle (dpy, root, DrawGC, x, y, i, 2);
+      XDrawRectangle (dpy, Scr.Root, DrawGC, x, y, i, 2);
       x += step / 2;
     }
   sleep_a_little (100000);
@@ -458,67 +480,58 @@ AnimateClose (int x, int y, int w, int h)
 }
 #endif
 
+/* error_handler:
+ * catch X errors, display the message and continue running.
+ */
+int
+error_handler (Display * disp, XErrorEvent * event)
+{
+  fprintf (stderr, "%s: internal error, error code %d, request code %d, minor code %d.\n",
+	 MyName, event->error_code, event->request_code, event->minor_code);
+  return 0;
+}
+
 void
 DeadPipe (int foo)
 {
   exit (0);
 }
 
+void
+usage (void)
+{
+  printf ("Usage:\n"
+	  "%s [--version] [--help]\n", MyName);
+  exit (0);
+}
+
 int
 main (int argc, char **argv)
 {
-  char *temp;
-  char *realconfigfile;
-  char configfile[255];
-  XGCValues gcv;
+  int i;
+  char *global_config_file = NULL;
   unsigned long color;
-  char mask_mesg[20];
+  XGCValues gcv;
+
+  /* Save our program name - for error messages */
+  SetMyName (argv[0]);
+
+  i = ProcessModuleArgs (argc, argv, &(global_config_file), NULL, NULL, usage);
 
-  temp = strrchr (argv[0], '/');
-  ProgName = temp ? temp + 1 : argv[0];
-  if (argc != 6)
-    {
-      fprintf (stderr, "%s Version %s should only be executed by afterstep!\n", ProgName, VERSION);
-      exit (1);
-    }
   /* Dead pipe == AS died */
   signal (SIGPIPE, DeadPipe);
+  signal (SIGQUIT, DeadPipe);
+  signal (SIGSEGV, DeadPipe);
+  signal (SIGTERM, DeadPipe);
 
-  Channel[0] = atoi (argv[1]);
-  Channel[1] = atoi (argv[2]);
+  x_fd = ConnectX (&Scr, display_name, PropertyChangeMask);
+  XSetErrorHandler (error_handler);
 
-  dpy = XOpenDisplay ("");
-  if (dpy == NULL)
-    {
-      fprintf (stderr, "%s: could not open display\n", ProgName);
-      exit (1);
-    }
-  root = DefaultRootWindow (dpy);
-  screen = DefaultScreen (dpy);
+  /* connect to AfterStep */
+  fd_width = GetFdWidth ();
+  fd[0] = fd[1] = ConnectAfterStep (mask_reg);
 
-  sprintf (mask_mesg, "SET_MASK %lu\n", (unsigned long) (M_CONFIGURE_WINDOW |
-							 M_ICONIFY |
-							 M_DEICONIFY |
-							 M_LOCKONSEND));
-
-  SendInfo (Channel, mask_mesg, 0);
-
-  if (strstr (argv[3], "steprc") != NULL)
-    ParseOptions (argv[3]);
-  else
-    {
-      sprintf (configfile, "%s/animate", AFTER_DIR);
-      realconfigfile = PutHome (configfile);
-
-      if ((CheckFile (realconfigfile)) == -1)
-	{
-	  free (realconfigfile);
-	  sprintf (configfile, "%s/animate", AFTER_SHAREDIR);
-	  realconfigfile = PutHome (configfile);
-	}
-      ParseOptions (realconfigfile);
-      free (realconfigfile);
-    }
+  LoadConfig (global_config_file, "animate", ParseOptions);
 
   color = WhitePixel (dpy, screen);
   if (Animate.color)
@@ -534,7 +547,7 @@ main (int argc, char **argv)
 	  if (color == 0)
 	    {
 	      fprintf (stderr, "\n%s: could not allocate color '%s' - using WHITE\n",
-		       ProgName, Animate.color);
+		       MyName, Animate.color);
 	      color = WhitePixel (dpy, screen);
 	    }
 
@@ -542,7 +555,7 @@ main (int argc, char **argv)
       else
 	{
 	  fprintf (stderr, "%s: could not parse color '%s'\n",
-		   ProgName, Animate.color);
+		   MyName, Animate.color);
 	}
     }
 
@@ -551,9 +564,13 @@ main (int argc, char **argv)
   gcv.foreground = color;
   gcv.background = color;
   gcv.subwindow_mode = IncludeInferiors;
-  DrawGC = XCreateGC (dpy, root, GCFunction | GCForeground | GCLineWidth | GCBackground
-		      | GCSubwindowMode, &gcv);
-  SendText (Channel, "Nop", 0);
+  DrawGC = XCreateGC (dpy, Scr.Root, GCFunction |
+		      GCForeground |
+		      GCLineWidth |
+		      GCBackground |
+		      GCSubwindowMode,
+		      &gcv);
+  SendInfo (fd, "Nop \"\"", 0);
   Loop ();
   return 0;
 }
@@ -575,51 +592,57 @@ GetWindowSize (Window win, int *x, int *y, int *w, int *h)
 void
 Loop ()
 {
-  unsigned long header[3], *body;
-  int c, x0, y0, w0, h0, xf, yf, wf, hf;
+  int x0, y0, w0, h0, xf, yf, wf, hf;
   Window win;
+  ASMessage *asmsg = NULL;
 
   while (1)
     {
-      c = ReadASPacket (Channel[1], header, &body);
-      if (c > 0)
+      asmsg = CheckASMessage (fd[1], -1);
+      if (asmsg)
 	{
-	  switch (header[1])
+	  switch (asmsg->header[1])
 	    {
 	    case M_DEICONIFY:
-	      win = body[1];
-	      x0 = body[3];
-	      y0 = body[4];
+	      win = (Window) * (asmsg->body + 1);
+	      x0 = (int) *(asmsg->body + 3);
+	      y0 = (int) *(asmsg->body + 4);
 
 	      GetWindowSize (win, &xf, &yf, &wf, &hf);
 	      Animate.resize (x0, y0, 64, 64, xf, yf, wf, hf);
+	      DestroyASMessage (asmsg);
 	      break;
 
 	    case M_ICONIFY:
-	      xf = body[3];
-	      yf = body[4];
+	      xf = (int) *(asmsg->body + 3);
+	      yf = (int) *(asmsg->body + 4);
 	      if (xf <= -10000)
 		break;
 	      wf = 64;
 	      hf = 64;
-	      free (body);
+	      DestroyASMessage (asmsg);
 
-	      SendInfo (Channel, "UNLOCK 1\n", 0);
-	      c = ReadASPacket (Channel[1], header, &body);
-	      if (c <= 0)
+	      SendInfo (fd, "UNLOCK 1\n", 0);
+	      asmsg = CheckASMessage (fd[1], -1);
+	      if (!asmsg)
 		break;
-	      if (header[1] != M_CONFIGURE_WINDOW)
-		break;
-	      x0 = body[3];
-	      y0 = body[4];
-	      w0 = body[5];
-	      h0 = body[6];
+	      if (asmsg->header[1] != M_CONFIGURE_WINDOW)
+		{
+		  DestroyASMessage (asmsg);
+		  break;
+		}
+	      x0 = (int) *(asmsg->body + 3);
+	      y0 = (int) *(asmsg->body + 4);
+	      w0 = (int) *(asmsg->body + 5);
+	      h0 = (int) *(asmsg->body + 6);
 	      Animate.resize (x0, y0, w0, h0, xf, yf, wf, hf);
+	      DestroyASMessage (asmsg);
 	      break;
+	    default:
+	      DestroyASMessage (asmsg);
 	    }
-	  free (body);
 	}
-      SendInfo (Channel, "UNLOCK 1\n", 0);
+      SendInfo (fd, "UNLOCK 1\n", 0);
     }
 }
 
@@ -657,15 +680,15 @@ ParseOptions (const char *filename)
 
   if ((fd = fopen (filename, "r")) == NULL)
     {
-      fprintf (stderr, "%s: can't open config file %s", ProgName, filename);
+      fprintf (stderr, "%s: can't open config file %s", MyName, filename);
       exit (1);
     }
   line = (char *) safemalloc (MAXLINELENGTH);
   parsebuf = (char *) safemalloc (MAXLINELENGTH);
-  len = strlen (ProgName);
+  len = strlen (MyName);
   while (fgets (line, MAXLINELENGTH, fd) != NULL)
     {
-      if ((*line == '*') && (!mystrncasecmp (line + 1, ProgName, len)))
+      if ((*line == '*') && (!mystrncasecmp (line + 1, MyName, len)))
 	{
 	  tline = line + len + 1;
 	  if (!mystrncasecmp (tline, "Color", 5))
@@ -714,7 +737,7 @@ ParseOptions (const char *filename)
 	      else
 		{
 		  Animate.resize = AnimateResizeFlip;
-		  fprintf (stderr, "%s: Bad Resize method '%s'\n", ProgName, parsebuf);
+		  fprintf (stderr, "%s: Bad Resize method '%s'\n", MyName, parsebuf);
 		}
 	    }
 	}
