@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1998 Sasha Vasko <sashav@sprintmail.com>
+ * Copyright (C) 1998 Sasha Vasko <sasha at aftercode.net>
  * Copyright (C) 1998 Makoto Kato <m_kato@ga2.so-net.ne.jp>
  * Copyright (C) 1996 Michael Beam
  * Copyright (C) 1996 Takanori Kubota
@@ -22,7 +22,6 @@
  */
 
 #include "../../configure.h"
-#include "../../include/aftersteplib.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,9 +34,6 @@
 #include <X11/cursorfont.h>
 #define XK_MISCELLANY
 #include <X11/keysymdef.h>
-#ifndef __FreeBSD__
-#include <malloc.h>
-#endif
 #include <string.h>
 #include <sys/types.h>
 #include <sys/time.h>
@@ -49,6 +45,9 @@
 #define XDrawImageString(t,u,v,w,x,y,z) XmbDrawImageString(t,u,FONTSET,v,w,x,y,z)
 #endif
 
+#include "../../include/aftersteplib.h"
+#include "../../include/module.h"
+
 void
 dummy ()
 {
@@ -170,8 +169,7 @@ Line;
 /* global variables */
 char *MyName;			/* program name, e.g. Form */
 
-int fd_in;			/* fd for Afterstep->Module packets */
-int fd_out;			/* fd for Module->Afterstep packets */
+int fd[2];			/* fd for Afterstep<->Module packets */
 int fd_err;
 FILE *fp_err;
 
@@ -239,7 +237,7 @@ copystring (char *cp, int n)
   char *dp, *bp;
   if (n == 0)
     n = strlen (cp);
-  bp = dp = (char *) malloc (n + 1);
+  bp = dp = (char *) safemalloc (n + 1);
   while (n-- > 0)
     *dp++ = *cp++;
   while (isspace ((unsigned char) *(--dp)));
@@ -259,7 +257,7 @@ char *
 CopyQuotedString (char *cp)
 {
   char *dp, *bp, c;
-  bp = dp = (char *) malloc (strlen (cp) + 1);
+  bp = dp = (char *) safemalloc (strlen (cp) + 1);
   while (1)
     {
       switch (c = *(cp++))
@@ -285,7 +283,7 @@ char *
 CopySolidString (char *cp)
 {
   char *dp, *bp, c;
-  bp = dp = (char *) malloc (strlen (cp) + 1);
+  bp = dp = (char *) safemalloc (strlen (cp) + 1);
   while (1)
     {
       c = *(cp++);
@@ -346,13 +344,13 @@ ParseOptions (char *file_name)
   lines[0].n = 0;
   lines[0].justify = L_CENTER;
   lines[0].size_x = lines[0].size_y = 0;
-  lines[0].items = (Item **) malloc (sizeof (Item *) * ITEMS_PER_LINE);
+  lines[0].items = (Item **) safemalloc (sizeof (Item *) * ITEMS_PER_LINE);
   cur_line = lines;
 
   /* default button is for initial functions */
   cur_button = &def_button;
   def_button.button.n = 0;
-  def_button.button.commands = (char **) malloc (sizeof (char *) * MAX_ITEMS);
+  def_button.button.commands = (char **) safemalloc (sizeof (char *) * MAX_ITEMS);
   def_button.button.key = IB_CONTINUE;
 
   /* default fonts in case the *FFont's are missing */
@@ -456,7 +454,7 @@ ParseOptions (char *file_name)
 	    {
 	      fprintf (fp_err, "can't get font %s\n", font_names[f_text]);
 	      fn_tmp_length = strlen (font_names[f_text]) + strlen (",-*--14-*");
-	      fn_tmp = malloc (fn_tmp_length + 1);
+	      fn_tmp = safemalloc (fn_tmp_length + 1);
 	      strcpy (fn_tmp, font_names[f_text]);
 	      strcat (fn_tmp, ",-*--14-*");
 	      fprintf (fp_err, "trying...%s\n", fn_tmp);
@@ -494,7 +492,7 @@ ParseOptions (char *file_name)
 	    {
 	      fprintf (fp_err, "can't get font %s\n", font_names[f_button]);
 	      fn_tmp_length = strlen (font_names[f_button]) + strlen (",-*--14-*");
-	      fn_tmp = malloc (fn_tmp_length + 1);
+	      fn_tmp = safemalloc (fn_tmp_length + 1);
 	      strcpy (fn_tmp, font_names[f_button]);
 	      strcat (fn_tmp, ",-*--14-*");
 	      fprintf (fp_err, "trying...%s\n", fn_tmp);
@@ -532,7 +530,7 @@ ParseOptions (char *file_name)
 	    {
 	      fprintf (fp_err, "can't get font %s\n", font_names[f_input]);
 	      fn_tmp_length = strlen (font_names[f_input]) + strlen (",-*--14-*");
-	      fn_tmp = malloc (fn_tmp_length + 1);
+	      fn_tmp = safemalloc (fn_tmp_length + 1);
 	      strcpy (fn_tmp, font_names[f_input]);
 	      strcat (fn_tmp, ",-*--14-*");
 	      fprintf (fp_err, "trying...%s\n", fn_tmp);
@@ -572,7 +570,7 @@ ParseOptions (char *file_name)
 	  else
 	    cur_line->justify = L_LEFTRIGHT;
 	  cur_line->n = 0;
-	  cur_line->items = (Item **) malloc (sizeof (Item *) * ITEMS_PER_LINE);
+	  cur_line->items = (Item **) safemalloc (sizeof (Item *) * ITEMS_PER_LINE);
 	  continue;
 	}
       else if (strncmp (cp, "Text", 4) == 0)
@@ -623,11 +621,11 @@ ParseOptions (char *file_name)
 	    item->input.init_value = CopyQuotedString (++cp);
 	  else
 	    item->input.init_value = "";
-	  item->input.blanks = (char *) malloc (item->input.size);
+	  item->input.blanks = (char *) safemalloc (item->input.size);
 	  for (j = 0; j < item->input.size; j++)
 	    item->input.blanks[j] = ' ';
 	  item->input.buf = strlen (item->input.init_value) + 1;
-	  item->input.value = (char *) malloc (item->input.buf);
+	  item->input.value = (char *) safemalloc (item->input.buf);
 	  item->header.size_x = FontWidth (xfs[f_input]) * item->input.size
 	    + 2 * TEXT_SPC + 2 * BOX_SPC;
 	  item->header.size_y = FontHeight (xfs[f_input]) + 3 * TEXT_SPC
@@ -654,7 +652,7 @@ ParseOptions (char *file_name)
 	    cur_sel->select.key = IS_SINGLE;
 	  cur_sel->select.n = 0;
 	  cur_sel->select.choices =
-	    (Item **) malloc (sizeof (Item *) * CHOICES_PER_SEL);
+	    (Item **) safemalloc (sizeof (Item *) * CHOICES_PER_SEL);
 	  continue;
 	}
       else if (strncmp (cp, "Choice", 6) == 0)
@@ -736,7 +734,7 @@ ParseOptions (char *file_name)
 	    item->button.keypress = -1;
 	  item->button.len = strlen (item->button.text);
 	  item->button.n = 0;
-	  item->button.commands = (char **) malloc (sizeof (char *) * MAX_ITEMS);
+	  item->button.commands = (char **) safemalloc (sizeof (char *) * MAX_ITEMS);
 	  item->header.size_y = FontHeight (xfs[f_button]) + 2 * TEXT_SPC
 	    + 2 * BOX_SPC;
 	  item->header.size_x = 2 * TEXT_SPC + 2 * BOX_SPC
@@ -1289,7 +1287,7 @@ ParseCommand (int dn, char *sp, char end, int *dn1, char **sp1)
 			{
 			  for (cp = item->input.value; *cp != '\0'; cp++)
 			    {
-			      if (*cp == '\"' || *cp == '\'' || *cp == '\\')
+			      if (!isalnum (*cp))
 				AddChar ('\\');
 			      AddChar (*cp);
 			    }
@@ -1363,12 +1361,12 @@ DoCommand (Item * cmd)
       fprintf (fp_err, "Final command[%d]: [%s]\n", k, buf);
 
       /* send command */
-      write (fd_out, &ref, sizeof (Window));
+      write (fd[1], &ref, sizeof (Window));
       len = strlen (buf);
-      write (fd_out, &len, sizeof (int));
-      write (fd_out, buf, len);
+      write (fd[1], &len, sizeof (int));
+      write (fd[1], buf, len);
       len = 1;
-      write (fd_out, &len, sizeof (int));
+      write (fd[1], &len, sizeof (int));
     }
 
   /* post-command */
@@ -1510,7 +1508,7 @@ ReadAfterstep ()
   static char buffer[32];
   int n;
 
-  n = read (fd_in, buffer, 32);
+  n = read (fd[0], buffer, 32);
   if (n == 0)
     {
       if (grab_server)
@@ -1907,13 +1905,13 @@ MainLoop ()
   while (1)
     {
       FD_ZERO (&fds);
-      FD_SET (fd_in, &fds);
+      FD_SET (fd[0], &fds);
       FD_SET (fd_x, &fds);
 
       XFlush (dpy);
       if (select (32, &fds, NULL, NULL, NULL) > 0)
 	{
-	  if (FD_ISSET (fd_in, &fds))
+	  if (FD_ISSET (fd[0], &fds))
 	    ReadAfterstep ();
 	  if (FD_ISSET (fd_x, &fds))
 	    ReadXServer ();
@@ -1921,6 +1919,20 @@ MainLoop ()
     }
 }
 
+void
+version (void)
+{
+  printf ("%s version %s\n", MyName, VERSION);
+  exit (0);
+}
+
+void
+usage (void)
+{
+  printf ("Usage:\n"
+    "%s [-v|--version] [-h|--help] [--window window-id] [script-name]\n", MyName);
+  exit (0);
+}
 
 /* main procedure */
 int
@@ -1929,9 +1941,11 @@ main (int argc, char **argv)
   char configfile[255];
   char *realconfigfile;
   FILE *fdopen ();
+  char *temp;
   int i;
+  char *global_config_file = NULL;
 
-  buf = (char *) malloc (N);	/* some kludge */
+  buf = (char *) safemalloc (N);	/* some kludge */
 
 #ifdef DEBUG
   fd_err = open (".FormErrors", O_WRONLY | O_CREAT, 0777);
@@ -1945,54 +1959,63 @@ main (int argc, char **argv)
   if (setlocale (LC_CTYPE, "") == NULL)
     fprintf (fp_err, "can't set locale\n");
 #endif
-  /* we get rid of the path from program name */
-  MyName = argv[0];
-  i = strlen (MyName);
-  while (MyName[--i] != '/' && i > 0);
-  if (i > 0)
-    MyName = MyName + (i + 1);
-  fprintf (fp_err, "%s started...\n", MyName);
 
-  if (argc < 6)
+  ref = None;
+
+  /* Save our program name - for error messages */
+  temp = strrchr (argv[0], '/');
+  MyName = temp ? temp + 1 : argv[0];
+
+  for (i = 1; i < argc && *argv[i] == '-'; i++)
     {
-#ifndef DEBUG
-      fprintf (stderr, "%s Version %s should only be executed by AfterStep!\n", MyName, VERSION);
-      exit (1);
-#else
-      fd_out = 1;
-      fd_in = 0;
-      ref = None;
-#endif
+      if (!strcmp (argv[i], "-h") || !strcmp (argv[i], "--help"))
+	usage ();
+      else if (!strcmp (argv[i], "-v") || !strcmp (argv[i], "--version"))
+	version ();
+      else if (!strcmp (argv[i], "-w") || !strcmp (argv[i], "--window"))
+	ref = strtol (argv[++i], NULL, 16);
+      else if (!strcmp (argv[i], "-c") || !strcmp (argv[i], "--context"))
+	i++;
+      else if (!strcmp (argv[i], "-f") && i + 1 < argc)
+	global_config_file = argv[++i];
     }
-  else
-    {
-      if (argc == 7)
-	MyName = argv[6];
-      fd_out = atoi (argv[1]);
-      fd_in = atoi (argv[2]);
-      ref = strtol (argv[4], NULL, 16);
-      if (ref == 0)
-	ref = None;
+
+  if (i < argc)
+    MyName = argv[i++];
+
 #ifdef DEBUG
-      fprintf (fp_err, "ref == %d\n", ref);
+  fprintf (fp_err, "ref == %d\n", ref);
 #endif
-    }
 
-  if (!(dpy = XOpenDisplay (NULL)))
+  if ((dpy = XOpenDisplay ("")) == NULL)
     {
-      fprintf (fp_err, "%s: can't open display.\n", MyName);
+      fprintf (stderr, "%s: couldn't open display %s\n",
+	       MyName, XDisplayName (""));
       exit (1);
     }
-  fd_x = XConnectionNumber (dpy);
-
   screen = DefaultScreen (dpy);
+
+  /* connect to AfterStep */
+  temp = module_get_socket_property (RootWindow (dpy, screen));
+  fd[0] = fd[1] = module_connect (temp);
+  XFree (temp);
+  if (fd[0] < 0)
+    {
+      fprintf (stderr, "%s: unable to establish connection to AfterStep\n", MyName);
+      exit (1);
+    }
+  temp = safemalloc (9 + strlen (MyName) + 1);
+  sprintf (temp, "SET_NAME %s", MyName);
+  SendInfo (fd, temp, None);
+  free (temp);
+
+  fd_x = XConnectionNumber (dpy);
   root = RootWindow (dpy, screen);
   scr_depth = DefaultDepth (dpy, screen);
   d_cmap = DefaultColormap (dpy, screen);
 
-
-  if (strstr (argv[3], "steprc") != NULL)
-    ParseOptions (argv[3]);
+  if (global_config_file != NULL)
+    ParseOptions (global_config_file);
   else
     {
       sprintf (configfile, "%s/forms", AFTER_DIR);
