@@ -31,12 +31,3 @@ void DeadPipe(int nonsense);
 void process_message(unsigned long type,unsigned long *body);
 void do_save(void);
 void list_new_page(unsigned long *body);
-
-#ifdef BROKEN_SUN_HEADERS
-#include "../../src/sun_headers.h"
-#endif
-
-
-#ifdef NEEDS_ALPHA_HEADER
-#include "../../src/alpha_header.h"
-#endif /* NEEDS_ALPHA_HEADER */
