@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1998 Sasha Vasko <sashav@sprintmail.com>
+ * Copyright (C) 1998 Sasha Vasko <sasha at aftercode.net>
  * Copyright (C) 1996 Frederic Cordier <cordier@cui.unige.ch>
  * Copyright (C) 1998 Guylhem Aznar <guylhem@oeil.qc.ca>
  *
@@ -91,6 +91,8 @@ Debug ()
 
 }
 
+void yyparse (void);
+
 /* Lecture du fichier contenant le scipt */
 void
 ReadConfig (char *ScriptName)
@@ -134,7 +136,7 @@ ParseOptions (char *filename)
   fd = fopen (filename, "r");
   if (fd == (FILE *) 0)
     {
-      fprintf (stderr, "%s: can't open config file %s", MyName, filename);
+      fprintf (stderr, "%s: can't open config file %s\n", MyName, filename);
       exit (1);
     }
   tline = fgets (line, (sizeof line) - 1, fd);
@@ -743,32 +745,82 @@ ReadASScriptArg (int argc, char **argv, int IsFather)
     }
 }
 
+void
+version (void)
+{
+  printf ("%s version %s\n", MyName, VERSION);
+  exit (0);
+}
+
+void
+usage (void)
+{
+  printf ("Usage:\n"
+	  "%s [-v|--version] [-h|--help] [--window window-id] script\n", MyName);
+  exit (0);
+}
+
 /* main procedure */
 int
 main (int argc, char **argv)
 {
   int IsFather;
-  int i;
   char mask_mesg[50];
   char *realconfigfile;
-  char *temp, *s;
+  char *temp;
+  int i, k;
+  char *global_config_file = NULL;
 
   /* Save our program name - for error messages */
-  temp = argv[0];
+  temp = strrchr (argv[0], '/');
+  MyName = temp ? temp + 1 : argv[0];
 
-  s = strrchr (argv[0], '/');
-  if (s != NULL)
-    temp = s + 1;
+  for (i = 1; i < argc && *argv[i] == '-'; i++)
+    {
+      if (!strcmp (argv[i], "-h") || !strcmp (argv[i], "--help"))
+	usage ();
+      else if (!strcmp (argv[i], "-v") || !strcmp (argv[i], "--version"))
+	version ();
+      else if (!strcmp (argv[i], "-w") || !strcmp (argv[i], "--window"))
+	ref = strtol (argv[++i], NULL, 16);
+      else if (!strcmp (argv[i], "-c") || !strcmp (argv[i], "--context"))
+	i++;
+      else if (!strcmp (argv[i], "-f") && i + 1 < argc)
+	global_config_file = argv[++i];
+    }
 
-  MyName = safemalloc (strlen (temp) + 1);
-  strcpy (MyName, temp);
+  /* Dead pipe == dead AfterStep */
+  signal (SIGPIPE, DeadPipe);
 
-  /* On determine si le script a un pere */
-  if (argc >= 8)
-    IsFather = (argv[7][0] != (char) 161);
-  else
-    IsFather = 1;
+  if ((dpy = XOpenDisplay ("")) == NULL)
+    {
+      fprintf (stderr, "%s: couldn't open display %s\n",
+	       MyName, XDisplayName (""));
+      exit (1);
+    }
+  screen = DefaultScreen (dpy);
 
+  /* connect to AfterStep */
+  temp = module_get_socket_property (RootWindow (dpy, screen));
+  fd[0] = fd[1] = module_connect (temp);
+  XFree (temp);
+  if (fd[0] < 0)
+    {
+      fprintf (stderr, "%s: unable to establish connection to AfterStep\n", MyName);
+      exit (1);
+    }
+  temp = safemalloc (9 + strlen (MyName) + 1);
+  sprintf (temp, "SET_NAME %s", MyName);
+  SendInfo (fd, temp, None);
+  free (temp);
+
+#ifdef I18N
+  if (!setlocale (LC_CTYPE, AFTER_LOCALE))
+    {
+      fprintf (stderr, "%s : cannot set locale.\n", MyName);
+      exit (1);
+    }
+#endif
 
   signal (SIGPIPE, DeadPipe);
   signal (SIGINT, DeadPipe);	/* cleanup on other ways of closing too */
@@ -776,48 +828,38 @@ main (int argc, char **argv)
   signal (SIGQUIT, DeadPipe);
   signal (SIGTERM, DeadPipe);
 
-  if (argc < 6)
-    {
-      fprintf (stderr, "%s must be started by AfterStep.\n", MyName);
-      exit (1);
-    }
-  else if (argc >= 7)
-    {
-      ScriptName = argv[6];
-      ref = strtol (argv[4], NULL, 16);
-      if (ref == 0)
-	ref = None;
-      fd[0] = atoi (argv[1]);
-      fd[1] = atoi (argv[2]);
+  if (i == argc)
+    usage ();
 
-      sprintf (mask_mesg, "SET_MASK %lu\n", (unsigned long) (M_NEW_DESK |
-		       M_END_WINDOWLIST | M_MAP | M_RES_NAME | M_RES_CLASS |
-							     M_WINDOW_NAME));
+  ScriptName = argv[i++];
 
-      SendInfo (fd, mask_mesg, 0);
+  /* On determine si le script a un pere */
+  IsFather = (i >= argc || *argv[i] != (char) 161);
+  i += !IsFather;
 
-      /* Enregistrement des arguments du script */
-      x11base = (X11base *) calloc (1, sizeof (X11base));
-      x11base->TabArg[0] = MyName;
-      for (i = 8 - IsFather; i < argc; i++)
-	x11base->TabArg[i - 7 + IsFather] = argv[i];
+  sprintf (mask_mesg, "SET_MASK %lu\n", (unsigned long)
+	   (M_NEW_DESK | M_END_WINDOWLIST | M_MAP | M_RES_NAME |
+	    M_RES_CLASS | M_WINDOW_NAME));
 
-    }
+  SendInfo (fd, mask_mesg, 0);
+
+  /* Enregistrement des arguments du script */
+  x11base = (X11base *) calloc (1, sizeof (X11base));
+  x11base->TabArg[0] = MyName;
+  for (k = 1; i < argc; i++, k++)
+    x11base->TabArg[k] = argv[i];
+
+  if (global_config_file != NULL)
+    ParseOptions (global_config_file);
   else
     {
-      fprintf (stderr, "%s requires only the path of the script.\n", MyName);
-      exit (1);
-    }
-
-  if (strstr (argv[3], "steprc") != NULL)
-    ParseOptions (argv[3]);
-  else
-    {
-      realconfigfile = (char *) PutHome (argv[3]);
+      char configfile[255];
+      sprintf (configfile, "%s/base.%dbpp", AFTER_DIR, DefaultDepth (dpy, screen));
+      realconfigfile = (char *) PutHome (configfile);
       ParseOptions (realconfigfile);
     }
 
-  SendText (fd, "Send_WindowList", 0);
+  SendInfo (fd, "Send_WindowList", 0);
 
   ReadConfig (ScriptName);	/* Lecture et analyse du script */
 
