@@ -39,7 +39,6 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <stdlib.h>
-#include "../../include/module.h"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -47,6 +46,7 @@
 #include <X11/Xatom.h>
 #include <X11/Intrinsic.h>
 
+#include "../../include/module.h"
 #include "Scroll.h"
 char *MyName;
 
@@ -226,99 +226,6 @@ CreateWindow (int x, int y, int w, int h)
 
 
 
-
-
-
-
-/****************************************************************************
- *
- * This routine computes the shadow color from the background color
- *
- ****************************************************************************/
-Pixel
-GetShadow (Pixel background)
-{
-  XColor bg_color;
-  XWindowAttributes attributes;
-
-  XGetWindowAttributes (dpy, Root, &attributes);
-
-  bg_color.pixel = background;
-  XQueryColor (dpy, attributes.colormap, &bg_color);
-
-  bg_color.red = (unsigned short) ((bg_color.red * 50) / 100);
-  bg_color.green = (unsigned short) ((bg_color.green * 50) / 100);
-  bg_color.blue = (unsigned short) ((bg_color.blue * 50) / 100);
-
-  if (!XAllocColor (dpy, attributes.colormap, &bg_color))
-    nocolor ("alloc shadow", "");
-
-  return bg_color.pixel;
-}
-
-/****************************************************************************
- *
- * This routine computes the hilight color from the background color
- *
- ****************************************************************************/
-Pixel
-GetHilite (Pixel background)
-{
-  XColor bg_color, white_p;
-  XWindowAttributes attributes;
-
-  XGetWindowAttributes (dpy, Root, &attributes);
-
-  bg_color.pixel = background;
-  XQueryColor (dpy, attributes.colormap, &bg_color);
-
-  white_p.pixel = GetColor ("white");
-  XQueryColor (dpy, attributes.colormap, &white_p);
-
-#ifndef min
-#define min(a,b) (((a)<(b)) ? (a) : (b))
-#define max(a,b) (((a)>(b)) ? (a) : (b))
-#endif
-
-  bg_color.red = max ((white_p.red / 5), bg_color.red);
-  bg_color.green = max ((white_p.green / 5), bg_color.green);
-  bg_color.blue = max ((white_p.blue / 5), bg_color.blue);
-
-  bg_color.red = min (white_p.red, (bg_color.red * 140) / 100);
-  bg_color.green = min (white_p.green, (bg_color.green * 140) / 100);
-  bg_color.blue = min (white_p.blue, (bg_color.blue * 140) / 100);
-
-  if (!XAllocColor (dpy, attributes.colormap, &bg_color))
-    nocolor ("alloc hilight", "");
-
-  return bg_color.pixel;
-}
-
-
-/****************************************************************************
- * 
- * Loads a single color
- *
- ****************************************************************************/
-Pixel
-GetColor (char *name)
-{
-  XColor color;
-  XWindowAttributes attributes;
-
-  XGetWindowAttributes (dpy, Root, &attributes);
-  color.pixel = 0;
-  if (!XParseColor (dpy, attributes.colormap, name, &color))
-    {
-      nocolor ("parse", name);
-    }
-  else if (!XAllocColor (dpy, attributes.colormap, &color))
-    {
-      nocolor ("alloc", name);
-    }
-  return color.pixel;
-}
-
 /***********************************************************************
  *
  *  Procedure:
