@@ -22,12 +22,3 @@ void RedrawLeftButton(GC rgc, GC sgc,int x1,int y1);
 void RedrawRightButton(GC rgc, GC sgc,int x1,int y1);
 void RedrawTopButton(GC rgc, GC sgc,int x1,int y1);
 void RedrawBottomButton(GC rgc, GC sgc,int x1,int y1);
-
-#ifdef BROKEN_SUN_HEADERS
-#include "../../src/sun_headers.h"
-#endif
-
-#ifdef NEEDS_ALPHA_HEADER
-#include "../../src/alpha_header.h"
-#endif /* NEEDS_ALPHA_HEADER */
-                                        
