@@ -23,8 +23,8 @@
  */
 
 /****************************************************************************
- * This module is based on Twm, but has been SIGNIFICANTLY modified 
- * by Rob Nation 
+ * This module is based on Twm, but has been SIGNIFICANTLY modified
+ * by Rob Nation
  * by Bo Yang
  * by Dong-hwa Oh <siage@nownuri.net>          ) korean support patch/
  * by Tomonori <manome@itlb.te.noda.sut.ac.jp> )
@@ -69,13 +69,21 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
 #include <X11/Xatom.h>
 
+#include "../../include/aftersteplib.h"
 #include "../../include/afterstep.h"
-#include "../../include/misc.h"
-#include "../../include/screen.h"
 #include "../../include/parse.h"
+#include "../../include/misc.h"
+#include "../../include/style.h"
+#include "../../include/screen.h"
 #include "../../include/module.h"
+#include "../../include/parser.h"
+
+#include "menus.h"
 
 #ifdef SHAPE
 #include <X11/extensions/shape.h>
@@ -85,21 +93,6 @@
 extern int LastWarpIndex;
 char NoName[] = "Untitled";	/* name if no name is specified */
 
-/* Used to parse command line of clients for specific desk requests. */
-/* Todo: check for multiple desks. */
-static XrmDatabase db;
-static XrmOptionDescRec table[] =
-{
-  /* Want to accept "-workspace N" or -xrm "afterstep*desk:N" as options
-   * to specify the desktop. I have to include dummy options that
-   * are meaningless since Xrm seems to allow -w to match -workspace
-   * if there would be no ambiguity. */
-  {"-workspacf", "*junk", XrmoptionSepArg, (caddr_t) NULL},
-  {"-workspace", "*desk", XrmoptionSepArg, (caddr_t) NULL},
-  {"-xrn", NULL, XrmoptionResArg, (caddr_t) NULL},
-  {"-xrm", NULL, XrmoptionResArg, (caddr_t) NULL},
-};
-
 void
 init_titlebar_windows (ASWindow * tmp_win, Bool free_resources)
 {
@@ -116,7 +109,7 @@ init_titlebar_windows (ASWindow * tmp_win, Bool free_resources)
 
 /*
  * Create the titlebar.
- * We place the window anywhere, and let SetupFrame() take care of 
+ * We place the window anywhere, and let SetupFrame() take care of
  *  putting it where it belongs.
  */
 Bool
@@ -127,10 +120,10 @@ create_titlebar_windows (ASWindow * tmp_win)
 
   if (!(tmp_win->flags & TITLE))
     return False;
-  valuemask = CWBackPixel | CWBorderPixel | CWCursor | CWEventMask;
-  /* these two colors don't make much sense, but they don't affect much either */
-  attributes.background_pixel = tmp_win->BackPixel;
-  attributes.border_pixel = tmp_win->TextPixel;
+
+  valuemask = CWBackPixmap | CWBorderPixel | CWCursor | CWEventMask;
+  attributes.background_pixmap = ParentRelative;
+  attributes.border_pixel = BlackPixel (dpy, Scr.screen);
   if (Scr.flags & BackingStore)
     {
       valuemask |= CWBackingStore;
@@ -161,11 +154,13 @@ init_titlebutton_windows (ASWindow * tmp_win, Bool free_resources)
 	{
 	  if (tmp_win->right_w[i] != None)
 	    {
+	      balloon_delete (balloon_find (tmp_win->left_w[i]));
 	      XDestroyWindow (dpy, tmp_win->left_w[i]);
 	      XDeleteContext (dpy, tmp_win->left_w[i], ASContext);
 	    }
 	  if (tmp_win->right_w[i] != None)
 	    {
+	      balloon_delete (balloon_find (tmp_win->left_w[i]));
 	      XDestroyWindow (dpy, tmp_win->right_w[i]);
 	      XDeleteContext (dpy, tmp_win->right_w[i], ASContext);
 	    }
@@ -182,9 +177,101 @@ init_titlebutton_windows (ASWindow * tmp_win, Bool free_resources)
   tmp_win->space_taken_right_buttons = 0;
 }
 
+/*
+   ** returns a newline delimited list of the Mouse functions bound to a
+   ** given context, in human readable form
+ */
+char *
+list_functions_by_context (int context)
+{
+  MouseButton *btn;
+  char *str = NULL;
+  int allocated_bytes = 0;
+
+  for (btn = Scr.MouseButtonRoot; btn != NULL; btn = btn->NextButton)
+    if (btn->Context & context)
+      {
+	extern SyntaxDef FuncSyntax;
+	TermDef *fterm;
+
+	fterm = FindTerm (&FuncSyntax, TT_ANY, btn->func);
+	if (fterm != NULL)
+	  {
+	    char *ptr;
+	    if (str)
+	      {
+		str = realloc (str, allocated_bytes + 1 + fterm->keyword_len + 32);
+		ptr = str + allocated_bytes;
+		*(ptr++) = '\n';
+	      }
+	    else
+	      ptr = str = safemalloc (fterm->keyword_len + 32);
+
+	    sprintf (ptr, "%s: ", fterm->keyword);
+	    ptr += fterm->keyword_len + 2;
+	    if (btn->Modifier & ShiftMask)
+	      {
+		strcat (ptr, "Shift+");
+		ptr += 8;
+	      }
+	    if (btn->Modifier & ControlMask)
+	      {
+		strcat (ptr, "Ctrl+");
+		ptr += 7;
+	      }
+	    if (btn->Modifier & Mod1Mask)
+	      {
+		strcat (ptr, "Meta+");
+		ptr += 7;
+	      }
+	    sprintf (ptr, "Button %d", btn->Button);
+	    allocated_bytes = strlen (str);
+	  }
+      }
+  return str;
+}
+
+/*
+   ** button argument:
+   ** 0 == leftmost button
+   ** 1 == rightmost button
+   ** 2 == next to leftmost button
+   ** 3 == next to rightmost button
+   ** etc.
+ */
+Bool
+create_titlebutton_balloon (ASWindow * tmp_win, int button)
+{
+  int n = button / 2 + 5 * (button & 1);
+  char *str;
+  if (!(tmp_win->flags & TITLE))
+    return False;
+  if (!(button & 1) && button / 2 < Scr.nr_left_buttons &&
+      Scr.button_style[button + 1] != NO_BUTTON_STYLE &&
+      !(tmp_win->buttons & (BUTTON1 << button)))
+    {
+      if ((str = list_functions_by_context (C_L1 << n)) != NULL)
+	{
+	  balloon_new_with_text (dpy, tmp_win->left_w[button / 2], str);
+	  free (str);
+	}
+    }
+  if ((button & 1) && button / 2 < Scr.nr_right_buttons &&
+      Scr.button_style[(button + 1) % 10] != NO_BUTTON_STYLE &&
+      !(tmp_win->buttons & (BUTTON1 << button)))
+    {
+      if ((str = list_functions_by_context (C_L1 << n)) != NULL)
+	{
+	  balloon_new_with_text (dpy, tmp_win->right_w[button / 2], str);
+	  free (str);
+	}
+    }
+  return True;
+}
+
 /*
  * Create the titlebar buttons.
- * We place the windows anywhere, and let SetupFrame() take care of 
+ * We place the windows anywhere, and let SetupFrame() take care of
  *  putting them where they belong.
  */
 Bool
@@ -195,10 +282,9 @@ create_titlebutton_windows (ASWindow * tmp_win)
   XSetWindowAttributes attributes;
   if (!(tmp_win->flags & TITLE))
     return False;
-  valuemask = CWBackPixel | CWBorderPixel | CWCursor | CWEventMask;
-  /* these two colors don't make much sense, but they don't affect much either */
-  attributes.background_pixel = tmp_win->BackPixel;
-  attributes.border_pixel = tmp_win->TextPixel;
+  valuemask = CWBackPixmap | CWBorderPixel | CWCursor | CWEventMask;
+  attributes.background_pixmap = ParentRelative;
+  attributes.border_pixel = BlackPixel (dpy, DefaultScreen (dpy));
   if (Scr.flags & BackingStore)
     {
       valuemask |= CWBackingStore;
@@ -225,6 +311,7 @@ create_titlebutton_windows (ASWindow * tmp_win)
 	  XSaveContext (dpy, tmp_win->left_w[i], ASContext, (caddr_t) tmp_win);
 	  tmp_win->nr_left_buttons++;
 	  tmp_win->space_taken_left_buttons += Scr.button_width[i + i + 1] + Scr.TitleButtonSpacing;
+	  create_titlebutton_balloon (tmp_win, i + i);
 	}
       if ((i < Scr.nr_right_buttons) &&
 	  (Scr.button_style[(i + i + 2) % 10] != NO_BUTTON_STYLE) &&
@@ -238,6 +325,7 @@ create_titlebutton_windows (ASWindow * tmp_win)
 	  XSaveContext (dpy, tmp_win->right_w[i], ASContext, (caddr_t) tmp_win);
 	  tmp_win->nr_right_buttons++;
 	  tmp_win->space_taken_right_buttons += Scr.button_width[(i + i + 2) % 10] + Scr.TitleButtonSpacing;
+	  create_titlebutton_balloon (tmp_win, i + i + 1);
 	}
     }
 
@@ -255,10 +343,6 @@ create_titlebutton_windows (ASWindow * tmp_win)
       tmp_win->space_taken_right_buttons += 6;
     }
 
-  i = tmp_win->frame_width;
-  tmp_win->frame_width = -1;	/* force reconfigure titlebar */
-  SetupFrame (tmp_win, tmp_win->frame_x, tmp_win->frame_y, i, tmp_win->frame_height, 0);
-  XMapSubwindows (dpy, tmp_win->frame);
   return True;
 }
 
@@ -285,42 +369,30 @@ AddWindow (Window w)
   int actual_format, i, width, height;
   unsigned long nitems, bytesafter;
   int a, b;
-  char *value;
-  unsigned long tflag;
-  int Desk, border_width = 0, resize_width = 0, viewport_x = -1, viewport_y = -1;
   extern Bool NeedToResizeToo;
   extern ASWindow *colormap_win;
-  char *forecolor = NULL, *backcolor = NULL;
-  int client_argc;
-  char **client_argv = NULL, *str_type;
-  Bool status;
-  XrmValue rm_value;
   XTextProperty text_prop;
-  Bool menu_window = False;	/* is w a menu window? */
-  MenuRoot *menu;
+  name_list nl;
 #ifdef I18N
   char **list;
   int num;
 #endif
-
   extern Bool PPosOverride;
 
-  for (menu = Scr.first_menu; menu != NULL; menu = (*menu).next)
-    if (w == (*menu).w)
-      menu_window = True;
-
-  NeedToResizeToo = False;
   /* allocate space for the afterstep window */
   tmp_win = (ASWindow *) calloc (1, sizeof (ASWindow));
+  if (tmp_win == (ASWindow *) 0)
+    return NULL;
+
+  NeedToResizeToo = False;
+
   init_titlebar_windows (tmp_win, False);
   init_titlebutton_windows (tmp_win, False);
-  if (tmp_win == (ASWindow *) 0)
-    {
-      return NULL;
-    }
+
   tmp_win->flags = VISIBLE;
   tmp_win->w = w;
   tmp_win->cmap_windows = (Window *) NULL;
+  tmp_win->layer = 0;
 
   if (XGetGeometry (dpy, tmp_win->w, &JunkRoot, &JunkX, &JunkY,
 		    &JunkWidth, &JunkHeight, &JunkBW, &JunkDepth) == 0)
@@ -353,6 +425,8 @@ AddWindow (Window w)
   else
     tmp_win->name = NoName;
 
+/*  fprintf( stderr, "[%s]: %dx%d%+d%+d\n", tmp_win->name, JunkWidth, JunkHeight, JunkX, JunkY );
+*/
   tmp_win->focus_sequence = 1;
   SetCirculateSequence (tmp_win, -1);
   tmp_win->class = NoClass;
@@ -364,7 +438,7 @@ AddWindow (Window w)
 
   FetchWmProtocols (tmp_win);
   FetchWmColormapWindows (tmp_win);
-  if (!(XGetWindowAttributes (dpy, tmp_win->w, &(tmp_win->attr))))
+  if (!XGetWindowAttributes (dpy, tmp_win->w, &tmp_win->attr))
     tmp_win->attr.colormap = Scr.ASRoot.attr.colormap;
 
   tmp_win->wmhints = XGetWMHints (dpy, tmp_win->w);
@@ -396,115 +470,145 @@ AddWindow (Window w)
    * If its a transient, and DecorateTransients was specified,
    *  decorate anyway
    */
+#ifndef NO_TEXTURE
   /*  Assume that we'll decorate */
-  tmp_win->flags |= BORDER;
+  if (DecorateFrames)
+    tmp_win->flags |= FRAME;
+  else
+#endif /* !NO_TEXTURE */
+    tmp_win->flags |= BORDER;
   tmp_win->flags |= TITLE;
 
-  tflag = LookInList (Scr.TheList, tmp_win->name, &tmp_win->class, &value, &Desk,
-		      &border_width, &resize_width,
-       &forecolor, &backcolor, &tmp_win->buttons, &viewport_x, &viewport_y);
+  style_init (&nl);
+
+  nl.off_buttons = tmp_win->buttons;
+  style_fill_by_name (&nl, tmp_win->name, NULL, tmp_win->class.res_name, tmp_win->class.res_class);
+  tmp_win->buttons = nl.off_buttons;
+
+  if (!(nl.off_flags & STYLE_FOCUS_FLAG) || (tmp_win->style_focus = mystyle_find (nl.style_focus)) == NULL)
+    tmp_win->style_focus = Scr.MSFWindow;
+  if (!(nl.off_flags & STYLE_UNFOCUS_FLAG) || (tmp_win->style_unfocus = mystyle_find (nl.style_unfocus)) == NULL)
+    tmp_win->style_unfocus = Scr.MSUWindow;
+  if (!(nl.off_flags & STYLE_STICKY_FLAG) || (tmp_win->style_sticky = mystyle_find (nl.style_sticky)) == NULL)
+    tmp_win->style_sticky = Scr.MSSWindow;
 
   GetMwmHints (tmp_win);
 
-  /* if this is a menu window, don't decorate */
-  if (menu_window == True)
-    tflag = NOBORDER_FLAG | NOTITLE_FLAG | STAYSONTOP_FLAG |
-      CIRCULATESKIP_FLAG | STICKY_FLAG;
+  SelectDecor (tmp_win, nl.off_flags, nl.border_width, nl.resize_width);
 
-  SelectDecor (tmp_win, tflag, border_width, resize_width);
-
-  if (viewport_x >= 0 || viewport_y >= 0)
+  if (nl.ViewportX >= 0 || nl.ViewportY >= 0)
     {
-      if (menu_window == True || (tflag & STICKY_FLAG))
-	viewport_x = viewport_y = -1;
+      if (nl.off_flags & STICKY_FLAG)
+	nl.ViewportX = nl.ViewportY = -1;
       else
 	{
-	  if (viewport_x < 0)
-	    viewport_x = Scr.Vx;
-	  if (viewport_y < 0)
-	    viewport_y = Scr.Vy;
+	  if (nl.ViewportX < 0)
+	    nl.ViewportX = Scr.Vx;
+	  if (nl.ViewportY < 0)
+	    nl.ViewportY = Scr.Vy;
 	  if (!PPosOverride)
-	    MoveViewport (viewport_x, viewport_y, 0);
+	    MoveViewport (nl.ViewportX, nl.ViewportY, 0);
 	}
     }
 
-  if (tflag & NOFOCUS_FLAG)
+  if (nl.off_flags & NOFOCUS_FLAG)
     tmp_win->focus_var = 1;
   else
     tmp_win->focus_var = 0;
-  if (tflag & START_ICONIC_FLAG)
+
+  if (nl.off_flags & NOFOCUS_FLAG)
+    tmp_win->flags |= NOFOCUS;
+  if (nl.off_flags & START_ICONIC_FLAG)
     tmp_win->flags |= STARTICONIC;
-  if (tflag & STAYSONTOP_FLAG)
-    tmp_win->flags |= ONTOP;
-  if (tflag & STAYSONBACK_FLAG)
-    tmp_win->flags |= ONBACK;
-  if (tflag & STICKY_FLAG)
+  if (nl.off_flags & LAYER_FLAG)
+    tmp_win->layer = nl.layer;
+  if (nl.off_flags & AVOID_COVER_FLAG)
+    tmp_win->flags |= AVOID_COVER;
+  if (nl.off_flags & VERTICAL_TITLE_FLAG)
+    tmp_win->flags |= VERTICAL_TITLE;
+  if (nl.off_flags & STICKY_FLAG)
     tmp_win->flags |= STICKY;
-  if (tflag & LISTSKIP_FLAG)
+  if (nl.off_flags & LISTSKIP_FLAG)
     tmp_win->flags |= WINDOWLISTSKIP;
-  if (tflag & CIRCULATESKIP_FLAG)
+  if (nl.off_flags & CIRCULATESKIP_FLAG)
     tmp_win->flags |= CIRCULATESKIP;
 
-  if (tflag & SUPPRESSICON_FLAG)
+  if (!(nl.off_flags & NOHANDLES_FLAG))
+    tmp_win->flags |= HANDLES;
+  else
+    tmp_win->flags &= ~FRAME;
+
+/*  if (!(nl.off_flags & NOFRAME_FLAG))
+   tmp_win->flags |= FRAME;     */
+
+  if (nl.off_flags & SUPPRESSICON_FLAG)
     tmp_win->flags |= SUPPRESSICON;
-#ifdef NO_ICON_BACKGROUND
-  if (tflag & NOICON_TITLE_FLAG)
+  if (nl.off_flags & NOICON_TITLE_FLAG)
     tmp_win->flags |= NOICON_TITLE;
-#endif
 
   if (Scr.flags & SuppressIcons)
     tmp_win->flags |= SUPPRESSICON;
 
   /* find a suitable icon pixmap */
-  if (tflag & ICON_FLAG)
+  if (nl.off_flags & ICON_FLAG)
     {
       /* an icon was specified */
-      tmp_win->icon_bitmap_file = value;
+      tmp_win->icon_pm_file = nl.icon_file;
     }
-  else if ((tmp_win->wmhints)
+  else if ((Scr.flags & KeepIconWindows) && (tmp_win->wmhints != NULL)
 	   && (tmp_win->wmhints->flags & (IconWindowHint | IconPixmapHint)))
     {
       /* window has its own icon */
-      tmp_win->icon_bitmap_file = NULL;
+      tmp_win->icon_pm_file = NULL;
     }
   else
     {
       /* use default icon */
-      tmp_win->icon_bitmap_file = Scr.DefaultIcon;
+      tmp_win->icon_pm_file = Scr.DefaultIcon;
     }
 
   GetWindowSizeHints (tmp_win);
 
+  if ( get_flags(nl.off_flags, PREPOS_FLAG) )
+  {
+      if( !get_flags(tmp_win->hints.flags, USPosition))
+	  {
+		  if( get_flags( nl.PreposFlags, XValue ) )
+		  {
+			  if( get_flags( nl.PreposFlags, XNegative ) )
+				  tmp_win->attr.x = Scr.MyDisplayWidth + nl.PreposX;
+			  else
+				  tmp_win->attr.x = nl.PreposX ;
+		  }
+		  if( get_flags( nl.PreposFlags, YValue ) )
+		  {
+			  if( get_flags( nl.PreposFlags, YNegative ) )
+			      tmp_win->attr.y = Scr.MyDisplayHeight + nl.PreposY;
+			  else
+		  		  tmp_win->attr.y = nl.PreposY;
+		  }
+	  }
+	  if( !get_flags(tmp_win->hints.flags, USSize))
+	  {
+		  if( get_flags( nl.PreposFlags, WidthValue ) )
+			  tmp_win->attr.width = nl.PreposWidth ;
+		  if( get_flags( nl.PreposFlags, HeightValue ) )
+			  tmp_win->attr.height = nl.PreposHeight ;
+	  }
+  }
   /* size and place the window */
-  tmp_win->frame_width = tmp_win->attr.width + 2 * tmp_win->boundary_width;
-  tmp_win->frame_height = tmp_win->attr.height + (tmp_win->title_height + tmp_win->bw) + tmp_win->boundary_height;
+  set_titlebar_geometry (tmp_win);
 
+  get_frame_geometry (tmp_win, tmp_win->attr.x, tmp_win->attr.y, tmp_win->attr.width, tmp_win->attr.height, NULL, NULL, &tmp_win->frame_width, &tmp_win->frame_height);
   ConstrainSize (tmp_win, &tmp_win->frame_width, &tmp_win->frame_height);
 
-  if (!PlaceWindow (tmp_win, tflag, Desk))
+  if (!PlaceWindow (tmp_win, nl.off_flags, nl.Desk))
     {
-      free ((char *) tmp_win);
+      Destroy (tmp_win, False);
       return NULL;
     }
 
-  tmp_win->frame_x = tmp_win->attr.x + tmp_win->old_bw - tmp_win->bw;
-  tmp_win->frame_y = tmp_win->attr.y + tmp_win->old_bw - tmp_win->bw;
-
-  /* Find out if the client requested a specific desk on the command line. */
-  if (XGetCommand (dpy, tmp_win->w, &client_argv, &client_argc))
-    {
-      XrmParseCommand (&db, table, 4, "afterstep", &client_argc, client_argv);
-      XFreeStringList (client_argv);
-      status = XrmGetResource (db, "afterstep.desk", "AS.Desk", &str_type, &rm_value);
-      if ((status == True) && (rm_value.size != 0))
-	{
-	  Desk = atoi (rm_value.addr);
-	  tmp_win->flags |= STAYSONDESK_FLAG;
-	  tflag |= STAYSONDESK_FLAG;
-	}
-      XrmDestroyDatabase (db);
-      db = NULL;
-    }
+  get_frame_geometry (tmp_win, tmp_win->attr.x, tmp_win->attr.y, tmp_win->attr.width, tmp_win->attr.height, &tmp_win->frame_x, &tmp_win->frame_y, &tmp_win->frame_width, &tmp_win->frame_height);
 
   /*
    * Make sure the client window still exists.  We don't want to leave an
@@ -541,7 +645,7 @@ AddWindow (Window w)
 	    tmp_win->icon_name = (char *) text_prop.value;
 	  else
 	    {
-	      if (XmbTextPropertyToTextList (dpy, &text_prop, &list, &num) > Success && num > 0 && *list)
+	      if (XmbTextPropertyToTextList (dpy, &text_prop, &list, &num) >= Success && num > 0 && *list)
 		tmp_win->icon_name = *list;
 	      else
 		tmp_win->icon_name = (char *) text_prop.value;
@@ -564,55 +668,6 @@ AddWindow (Window w)
   tmp_win->flags &= ~ICON_UNMAPPED;
   tmp_win->flags &= ~MAXIMIZED;
 
-#ifdef NEWLOOK
-  tmp_win->TextPixel = (*Scr.MSFWindow).colors.fore;
-  tmp_win->BackPixel = (*Scr.MSFWindow).colors.back;
-  tmp_win->ReliefPixel = (*Scr.MSFWindow).relief.fore;
-  tmp_win->ShadowPixel = (*Scr.MSFWindow).relief.back;
-#else
-  tmp_win->TextPixel = Scr.StdColors.fore;
-  tmp_win->ReliefPixel = Scr.StdRelief.fore;
-  tmp_win->ShadowPixel = Scr.StdRelief.back;
-  tmp_win->BackPixel = Scr.StdColors.back;
-#endif
-
-  if (tmp_win->flags & STICKY)
-    {
-#ifdef NEWLOOK
-      tmp_win->BackPixel = (*Scr.MSSWindow).colors.back;
-      tmp_win->ShadowPixel = (*Scr.MSSWindow).relief.back;
-      tmp_win->ReliefPixel = (*Scr.MSSWindow).relief.fore;
-      tmp_win->TextPixel = (*Scr.MSSWindow).colors.fore;
-#else
-      tmp_win->BackPixel = Scr.StickyColors.back;
-      tmp_win->ShadowPixel = Scr.StickyRelief.back;
-      tmp_win->ReliefPixel = Scr.StickyRelief.fore;
-      tmp_win->TextPixel = Scr.StickyColors.fore;
-#endif
-    }
-  if (forecolor != NULL)
-    {
-      XColor color;
-
-      if ((XParseColor (dpy, Scr.ASRoot.attr.colormap, forecolor, &color))
-	  && (XAllocColor (dpy, Scr.ASRoot.attr.colormap, &color)))
-	{
-	  tmp_win->TextPixel = color.pixel;
-	}
-    }
-  if (backcolor != NULL)
-    {
-      XColor color;
-
-      if ((XParseColor (dpy, Scr.ASRoot.attr.colormap, backcolor, &color))
-	  && (XAllocColor (dpy, Scr.ASRoot.attr.colormap, &color)))
-	{
-	  tmp_win->BackPixel = color.pixel;
-	}
-      tmp_win->ShadowPixel = GetShadow (tmp_win->BackPixel);
-      tmp_win->ReliefPixel = GetHilite (tmp_win->BackPixel);
-    }
-
   /* the newly created window will be on the top of the warp list */
   tmp_win->warp_index = 0;
   ChangeWarpIndex (++LastWarpIndex, F_WARP_F);
@@ -636,10 +691,10 @@ AddWindow (Window w)
     }
   else
     {
-      attributes.background_pixel = tmp_win->BackPixel;
-      valuemask |= CWBackPixel;
+      attributes.background_pixmap = ParentRelative;
+      valuemask |= CWBackPixmap;
     }
-  attributes.border_pixel = tmp_win->TextPixel;
+  attributes.border_pixel = BlackPixel (dpy, DefaultScreen (dpy));
 
   attributes.cursor = Scr.ASCursors[DEFAULT];
   attributes.event_mask = (SubstructureRedirectMask | ButtonPressMask |
@@ -662,13 +717,13 @@ AddWindow (Window w)
   /* Thats not all, we'll double-reparent the window ! */
   attributes.cursor = Scr.ASCursors[DEFAULT];
   tmp_win->Parent =
-    XCreateWindow (dpy, tmp_win->frame,
-		   0, (tmp_win->title_height + tmp_win->bw),
-		   tmp_win->frame_width,
-	    (tmp_win->frame_height - (tmp_win->title_height + tmp_win->bw) -
-	     tmp_win->boundary_height), tmp_win->bw, CopyFromParent,
+    XCreateWindow (dpy, tmp_win->frame, -999, -999, 16, 16,
+		   tmp_win->bw, CopyFromParent,
 		   InputOutput, CopyFromParent, valuemask, &attributes);
 
+  XSetWindowBackgroundPixmap (dpy, tmp_win->frame, ParentRelative);
+  XSetWindowBackgroundPixmap (dpy, tmp_win->Parent, ParentRelative);
+
   if (Scr.flags & BackingStore)
     {
       valuemask |= CWBackingStore;
@@ -676,10 +731,9 @@ AddWindow (Window w)
     }
   attributes.event_mask = (ButtonPressMask | ButtonReleaseMask | ExposureMask |
 			   EnterWindowMask | LeaveWindowMask);
-  tmp_win->title_x = tmp_win->title_y = 0;
   tmp_win->title_w = 0;
-  /* title width can be anything non-zero - SetupFrame() will fix it */
-  tmp_win->title_width = 16;
+  /* title geometry can be anything - SetupFrame() will fix it */
+  tmp_win->title_x = tmp_win->title_y = -999;
   if (tmp_win->flags & BORDER)
     {
       /* Just dump the windows any old place and left SetupFrame take
@@ -689,13 +743,11 @@ AddWindow (Window w)
 	  attributes.cursor = Scr.ASCursors[BOTTOM_LEFT + i];
 	  tmp_win->corners[i] =
 	    XCreateWindow (dpy, tmp_win->frame, 0, 0,
-			   tmp_win->corner_width, tmp_win->corner_width,
+			   tmp_win->corner_width, tmp_win->boundary_height,
 			   0, CopyFromParent, InputOutput,
 			   CopyFromParent, valuemask, &attributes);
 	}
     }
-  create_titlebar_windows (tmp_win);
-  create_titlebutton_windows (tmp_win);
   if (tmp_win->flags & BORDER)
     {
       attributes.cursor = Scr.ASCursors[BOTTOM];
@@ -705,6 +757,12 @@ AddWindow (Window w)
 		       InputOutput, CopyFromParent, valuemask,
 		       &attributes);
     }
+  create_titlebar_windows (tmp_win);
+  create_titlebutton_windows (tmp_win);
+#ifndef NO_TEXTURE
+  frame_create_windows (tmp_win);
+#endif /* !NO_TEXTURE */
+
   XMapSubwindows (dpy, tmp_win->frame);
   XRaiseWindow (dpy, tmp_win->Parent);
   XReparentWindow (dpy, tmp_win->w, tmp_win->Parent, 0, 0);
@@ -721,17 +779,11 @@ AddWindow (Window w)
     }
   XChangeWindowAttributes (dpy, tmp_win->w, valuemask, &attributes);
 
-  if (menu_window == False)
-    XAddToSaveSet (dpy, tmp_win->w);
+  XAddToSaveSet (dpy, tmp_win->w);
 
-  tmp_win->iconPixmap = None;
-  tmp_win->icon_maskPixmap = None;
+  tmp_win->icon_pm_pixmap = None;
+  tmp_win->icon_pm_mask = None;
 
-#ifndef NO_TEXTURE
-  tmp_win->backPixmap = None;
-  tmp_win->backPixmap2 = None;
-  tmp_win->backPixmap3 = None;
-#endif
   /*
    * Reparenting generates an UnmapNotify event, followed by a MapNotify.
    * Set the map state to FALSE to prevent a transition back to
@@ -739,18 +791,23 @@ AddWindow (Window w)
    * again in HandleMapNotify.
    */
   tmp_win->flags &= ~MAPPED;
+
   width = tmp_win->frame_width;
-  tmp_win->frame_width = 0;
   height = tmp_win->frame_height;
+  tmp_win->frame_width = 0;
   tmp_win->frame_height = 0;
+
+#ifndef NO_TEXTURE
+  tmp_win->backPixmap = None;
+  tmp_win->backPixmap2 = None;
+  tmp_win->backPixmap3 = None;
+#endif
   SetupFrame (tmp_win, tmp_win->frame_x, tmp_win->frame_y, width, height, True);
 
   /* wait until the window is iconified and the icon window is mapped
    * before creating the icon window 
    */
-#ifdef NO_ICON_BACKGROUND
-  tmp_win->icon_w = None;
-#endif
+  tmp_win->icon_title_w = None;
   GrabButtons (tmp_win);
   GrabKeys (tmp_win);
   XSaveContext (dpy, tmp_win->w, ASContext, (caddr_t) tmp_win);
@@ -771,8 +828,7 @@ AddWindow (Window w)
 		&JunkWidth, &JunkHeight, &JunkBW, &JunkDepth);
   XTranslateCoordinates (dpy, tmp_win->frame, Scr.Root, JunkX, JunkY,
 			 &a, &b, &JunkChild);
-  tmp_win->xdiff -= a;
-  tmp_win->ydiff -= b;
+
   if (Scr.flags & ClickToFocus)
     {
       /* need to grab all buttons for window that we are about to
@@ -786,13 +842,6 @@ AddWindow (Window w)
 	  }
     }
   BroadcastConfig (M_ADD_WINDOW, tmp_win);
-/* added by Sasha Vasko to notify pager about new win before changing desktops */
-  if (!PPosOverride)
-    {
-      changeDesks (0, tmp_win->Desk);
-      if (viewport_x >= 0 && viewport_y >= 0)
-	MoveViewport (viewport_x, viewport_y, 0);
-    }
 
   BroadcastName (M_WINDOW_NAME, tmp_win->w, tmp_win->frame,
 		 (unsigned long) tmp_win, tmp_win->name);
@@ -879,7 +928,7 @@ void
 GrabKeys (ASWindow * tmp_win)
 {
   FuncKey *tmp;
-  for (tmp = Scr.FuncKeyRoot.next; tmp != NULL; tmp = tmp->next)
+  for (tmp = Scr.FuncKeyRoot; tmp != NULL; tmp = tmp->next)
     {
       if (tmp->cont & (C_WINDOW | C_TITLE | C_RALL | C_LALL | C_SIDEBAR))
 	MyXGrabKey (dpy, tmp->keycode, tmp->mods, tmp_win->frame, True,
@@ -1024,121 +1073,3 @@ GetWindowSizeHints (ASWindow * tmp)
       tmp->hints.flags |= PWinGravity;
     }
 }
-
-
-/***********************************************************************
- *
- *  Procedure:
- *	LookInList - look through a list for a window name, or class
- *
- *  Returned Value:
- *	the ptr field of the list structure or NULL if the name 
- *	or class was not found in the list
- *
- *  Inputs:
- *	list	- a pointer to the head of a list
- *	name	- a pointer to the name to look for
- *	class	- a pointer to the class to look for
- *
- ***********************************************************************/
-unsigned long
-LookInList (name_list * list, char *name, XClassHint * class,
-	    char **value, int *Desk, int *border_width,
-	    int *resize_width, char **forecolor, char **backcolor,
-	    unsigned long *buttons, int *viewport_x, int *viewport_y)
-{
-  name_list *nptr;
-  unsigned long retval = 0;
-
-  *value = NULL;
-  *forecolor = NULL;
-  *backcolor = NULL;
-  *Desk = 0;
-  *buttons = 0;
-  /* look for the name first */
-  if (!list)
-    return retval;
-  nptr = list;
-  do
-    {
-
-      if (class)
-	{
-	  /* first look for the res_class  (lowest priority) */
-	  if (matchWildcards (nptr->name, class->res_class) == TRUE)
-	    {
-	      if (nptr->value != NULL)
-		*value = nptr->value;
-	      if (nptr->off_flags & STAYSONDESK_FLAG)
-		*Desk = nptr->Desk;
-	      if (nptr->off_flags & VIEWPORTX_FLAG)
-		*viewport_x = nptr->ViewportX;
-	      if (nptr->off_flags & VIEWPORTY_FLAG)
-		*viewport_y = nptr->ViewportY;
-	      if (nptr->off_flags & BW_FLAG)
-		*border_width = nptr->border_width;
-	      if (nptr->off_flags & FORE_COLOR_FLAG)
-		*forecolor = nptr->ForeColor;
-	      if (nptr->off_flags & BACK_COLOR_FLAG)
-		*backcolor = nptr->BackColor;
-	      if (nptr->off_flags & NOBW_FLAG)
-		*resize_width = nptr->resize_width;
-	      retval |= nptr->off_flags;
-	      retval &= ~(nptr->on_flags);
-	      *buttons |= nptr->off_buttons;
-	      *buttons &= ~(nptr->on_buttons);
-	    }
-	  /* look for the res_name next */
-	  if (matchWildcards (nptr->name, class->res_name) == TRUE)
-	    {
-	      if (nptr->value != NULL)
-		*value = nptr->value;
-	      if (nptr->off_flags & STAYSONDESK_FLAG)
-		*Desk = nptr->Desk;
-	      if (nptr->off_flags & VIEWPORTX_FLAG)
-		*viewport_x = nptr->ViewportX;
-	      if (nptr->off_flags & VIEWPORTY_FLAG)
-		*viewport_y = nptr->ViewportY;
-	      if (nptr->off_flags & FORE_COLOR_FLAG)
-		*forecolor = nptr->ForeColor;
-	      if (nptr->off_flags & BACK_COLOR_FLAG)
-		*backcolor = nptr->BackColor;
-	      if (nptr->off_flags & BW_FLAG)
-		*border_width = nptr->border_width;
-	      if (nptr->off_flags & NOBW_FLAG)
-		*resize_width = nptr->resize_width;
-	      retval |= nptr->off_flags;
-	      retval &= ~(nptr->on_flags);
-	      *buttons |= nptr->off_buttons;
-	      *buttons &= ~(nptr->on_buttons);
-	    }
-	}
-      /* finally, look for name matches */
-      if (matchWildcards (nptr->name, name) == TRUE)
-	{
-	  if (nptr->value != NULL)
-	    *value = nptr->value;
-	  if (nptr->off_flags & STAYSONDESK_FLAG)
-	    *Desk = nptr->Desk;
-	  if (nptr->off_flags & VIEWPORTX_FLAG)
-	    *viewport_x = nptr->ViewportX;
-	  if (nptr->off_flags & VIEWPORTY_FLAG)
-	    *viewport_y = nptr->ViewportY;
-	  if (nptr->off_flags & FORE_COLOR_FLAG)
-	    *forecolor = nptr->ForeColor;
-	  if (nptr->off_flags & BACK_COLOR_FLAG)
-	    *backcolor = nptr->BackColor;
-	  if (nptr->off_flags & BW_FLAG)
-	    *border_width = nptr->border_width;
-	  if (nptr->off_flags & NOBW_FLAG)
-	    *resize_width = nptr->resize_width;
-	  retval |= nptr->off_flags;
-	  retval &= ~(nptr->on_flags);
-	  *buttons |= nptr->off_buttons;
-	  *buttons &= ~(nptr->on_buttons);
-	}
-      nptr = nptr->next;
-    }
-  while (nptr != list);
-  return retval;
-}
