@@ -1,6 +1,7 @@
 /****************************************************************************
- * This module is based on Twm, but has been SIGNIFICANTLY modified 
- * by Rob Nation 
+ * Copyright (c) 1999 Sasha Vasko <sasha at aftercode.net>
+ * This module is based on Twm, but has been SIGNIFICANTLY modified
+ * by Rob Nation
  * by Bo Yang
  * by Frank Fejes
  * by Alfredo Kojima
@@ -42,16 +43,21 @@
 #include <fcntl.h>
 #include <string.h>
 #include <unistd.h>
+
 #include <X11/Xlib.h>
 #include <X11/Xproto.h>
 #include <X11/Xatom.h>
 /* need to get prototype for XrmUniqueQuark for XUniqueContext call */
 #include <X11/Xresource.h>
 #include <X11/Xutil.h>
+#include <X11/cursorfont.h>
 
 #ifdef SHAPE
 #include <X11/extensions/shape.h>
 #endif /* SHAPE */
+#ifdef HAVE_XINERAMA
+#include <X11/extensions/Xinerama.h>
+#endif /* SHAPE */
 
 #ifdef I18N
 #include <X11/Xlocale.h>
@@ -62,11 +68,17 @@
 #include <sys/systeminfo.h>
 #endif
 
+#include "../../include/aftersteplib.h"
 #include "../../include/afterstep.h"
-#include "../../include/menus.h"
-#include "../../include/misc.h"
-#include "../../include/screen.h"
 #include "../../include/parse.h"
+#include "../../include/misc.h"
+#include "../../include/style.h"
+#include "../../include/screen.h"
+#include "../../include/module.h"
+#include "../../include/loadimg.h"
+
+#include "globals.h"
+#include "menus.h"
 
 #define MAXHOSTNAME 255
 
@@ -118,13 +130,16 @@ static char s_g_bits[] =
 #ifdef SHAPE
 int ShapeEventBase, ShapeErrorBase;
 #endif
+#ifdef HAVE_XINERAMA
+int XineEventBase, XineErrorBase;
+void get_Xinerama_rectangles(ScreenInfo *scr);
+#endif
 
 long isIconicState = 0;
 extern XEvent Event;
 extern Window XmuClientWindow ();
 Bool Restarting = False;
 int fd_width, x_fd;
-char *display_name = NULL;
 FILE *savewindow_fd;
 int SmartCircCounter = 0;	/* F_WARP_F/B 0=CirculateUp ; 1=CirculateDown */
 int LastFunction;
@@ -133,6 +148,30 @@ Bool single = False;
 static char *display_string;
 static char *rdisplay_string;
 
+ASDirs as_dirs =
+{NULL, NULL, NULL};
+
+void
+InitASDirNames (Bool freemem)
+{
+
+  if (freemem)
+    {
+      if (as_dirs.afters_noncfdir)
+	free (as_dirs.afters_noncfdir);
+      if (as_dirs.after_dir)
+	free (as_dirs.after_dir);
+      if (as_dirs.after_sharedir)
+	free (as_dirs.after_sharedir);
+    }
+  else
+    {
+      as_dirs.afters_noncfdir = PutHome (AFTER_NONCF);
+      as_dirs.after_dir = PutHome (AFTER_DIR);
+      as_dirs.after_sharedir = PutHome (AFTER_SHAREDIR);
+    }
+}
+
 /***********************************************************************
  *
  *  Procedure:
@@ -146,7 +185,6 @@ main (int argc, char **argv)
   XSetWindowAttributes attributes;	/* attributes for create windows */
   void InternUsefulAtoms (void);
   void InitVariables (int);
-  void enterAlarm (int);
   int i;
   extern int x_fd;
   int len;
@@ -159,12 +197,20 @@ main (int argc, char **argv)
   if (setlocale (LC_CTYPE, AFTER_LOCALE) == NULL)
     afterstep_err ("can't set locale", NULL, NULL, NULL);
 #endif
-
+  memset( &Scr, 0x00, sizeof(Scr));
   init_old_look_variables (False);
   InitBase (False);
   InitLook (False);
   InitFeel (False);
   InitDatabase (False);
+#ifndef NO_TEXTURE
+  frame_init (False);
+#endif /* !NO_TEXTURE */
+  module_init (0);
+
+#if defined(LOG_FONT_CALLS)
+  fprintf (stderr, "logging font calls now\n");
+#endif
 
   for (i = 1; i < argc; i++)
     {
@@ -208,7 +254,13 @@ main (int argc, char **argv)
   signal (SIGUSR1, Restart);
 
   signal (SIGPIPE, DeadPipe);
-  signal (SIGALRM, enterAlarm);
+
+#if 1				/* see SetTimer() */
+  {
+    void enterAlarm (int);
+    signal (SIGALRM, enterAlarm);
+  }
+#endif /* 1 */
 
   ReapChildren ();
 
@@ -225,7 +277,9 @@ main (int argc, char **argv)
       afterstep_err ("close-on-exec failed", NULL, NULL, NULL);
       exit (1);
     }
+
   Scr.screen = DefaultScreen (dpy);
+  screen = Scr.screen ;
   Scr.NumberOfScreens = ScreenCount (dpy);
 
   if (!single)
@@ -249,11 +303,15 @@ main (int argc, char **argv)
 		}
 	      strcat (message, " &\n");
 
-	      if (fork())
+	      if (!fork ())
 		execl ("/bin/sh", "sh", "-c", message, (char *) 0);
 	    }
 	}
     }
+
+  /* initializing our dirs names */
+  InitASDirNames (False);
+
   /*  Add a DISPLAY entry to the environment, incase we were started
    * with afterstep -display term:0.0
    */
@@ -296,6 +354,10 @@ main (int argc, char **argv)
 #ifdef SHAPE
   XShapeQueryExtension (dpy, &ShapeEventBase, &ShapeErrorBase);
 #endif /* SHAPE */
+#ifdef HAVE_XINERAMA
+  if( XineramaQueryExtension(dpy, &XineEventBase, &XineErrorBase))
+  	  get_Xinerama_rectangles(&Scr);
+#endif /* XINERAMA */
 
   InternUsefulAtoms ();
 
@@ -303,6 +365,8 @@ main (int argc, char **argv)
   XChangeProperty (dpy, Scr.Root, _XA_MIT_PRIORITY_COLORS,
 		   XA_CARDINAL, 32, PropModeReplace, NULL, 0);
 
+  set_gnome_proxy ();
+
   XSetErrorHandler ((XErrorHandler) CatchRedirectError);
   XSelectInput (dpy, Scr.Root,
 		LeaveWindowMask | EnterWindowMask | PropertyChangeMask |
@@ -314,13 +378,15 @@ main (int argc, char **argv)
 
   CreateCursors ();
   InitVariables (1);
-  initModules ();
+  module_setup ();
+#ifndef DONT_GRAB_SERVER
   XGrabServer (dpy);
+#endif
 
   Scr.gray_bitmap =
     XCreateBitmapFromData (dpy, Scr.Root, g_bits, g_width, g_height);
 
-  /* the SizeWindow will be moved into place in MakeMenus() */
+  /* the SizeWindow will be moved into place in LoadASConfig() */
   attributes.override_redirect = True;
   attributes.bit_gravity = NorthWestGravity;
   Scr.SizeWindow = XCreateWindow (dpy, Scr.Root, -999, -999, 10, 10, 0, 0,
@@ -328,40 +394,26 @@ main (int argc, char **argv)
 			    CWBitGravity | CWOverrideRedirect, &attributes);
 
   /* read config file, set up menus, colors, fonts */
-#ifdef NO_MAKEMENUSONBOOT
-  MakeMenus (display_name, 0, 0, 1, 1);
-#else
-  MakeMenus (display_name, 0, 1, 1, 1);
-#endif
+  LoadASConfig (display_name, 0, 1, 1, 1);
 
+/*print_unfreed_mem();
+ */
   if (Scr.d_depth < 2)
     {
       Scr.gray_pixmap =
 	XCreatePixmapFromBitmapData (dpy, Scr.Root, g_bits, g_width, g_height,
-#ifdef NEWLOOK
-		 (*Scr.MSFWindow).colors.fore, (*Scr.MSFWindow).colors.back,
-#else
-				     Scr.StdColors.fore, Scr.StdColors.back,
-#endif
+		 WhitePixel (dpy, Scr.screen), BlackPixel (dpy, Scr.screen),
 				     Scr.d_depth);
       Scr.light_gray_pixmap =
 	XCreatePixmapFromBitmapData (dpy, Scr.Root, l_g_bits, l_g_width, l_g_height,
-#ifdef NEWLOOK
-		 (*Scr.MSFWindow).colors.fore, (*Scr.MSFWindow).colors.back,
-#else
-				     Scr.StdColors.fore, Scr.StdColors.back,
-#endif
+		 WhitePixel (dpy, Scr.screen), BlackPixel (dpy, Scr.screen),
 				     Scr.d_depth);
       Scr.sticky_gray_pixmap =
 	XCreatePixmapFromBitmapData (dpy, Scr.Root, s_g_bits, s_g_width, s_g_height,
-#ifdef NEWLOOK
-		 (*Scr.MSFWindow).colors.fore, (*Scr.MSFWindow).colors.back,
-#else
-			       Scr.StickyColors.fore, Scr.StickyColors.back,
-#endif
+		 WhitePixel (dpy, Scr.screen), BlackPixel (dpy, Scr.screen),
 				     Scr.d_depth);
     }
-  /* create a window which will accept the keyboard focus when no other 
+  /* create a window which will accept the keyboard focus when no other
      windows have it */
   attributes.event_mask = KeyPressMask | FocusChangeMask;
   attributes.override_redirect = True;
@@ -384,9 +436,13 @@ main (int argc, char **argv)
 #ifndef NO_VIRTUAL
   checkPanFrames ();
 #endif /* NO_VIRTUAL */
+#ifndef DONT_GRAB_SERVER
   XUngrabServer (dpy);
+#endif
   fd_width = GetFdWidth ();
 
+  /* watch for incoming module connections */
+  module_setup_socket (Scr.Root, display_string);
 
   if (Restarting)
     {
@@ -401,6 +457,10 @@ main (int argc, char **argv)
 			 0, 0, Scr.InitFunction, -1);
     }
   XDefineCursor (dpy, Scr.Root, Scr.ASCursors[DEFAULT]);
+
+  /* make sure we're on the right desk, and the _WIN_DESK property is set */
+  changeDesks (0, Scr.CurrentDesk);
+
   HandleEvents ();
   return (0);
 }
@@ -457,7 +517,7 @@ CaptureAllWindows (void)
 
 
   /*
-   * map all of the non-override, non-menu windows (menus are handled 
+   * map all of the non-override, non-menu windows (menus are handled
    * elsewhere)
    */
 
@@ -540,13 +600,17 @@ Atom _XA_WM_COLORMAP_WINDOWS;
 Atom _XA_WM_PROTOCOLS;
 Atom _XA_WM_TAKE_FOCUS;
 Atom _XA_WM_DELETE_WINDOW;
-Atom _XA_WM_DESKTOP;
 Atom _XA_MwmAtom;
+Atom _XA_WIN_STATE;
+Atom _XROOTPMAP_ID;
+Atom _AS_STYLE;
+Atom _AS_MODULE_SOCKET;
+Atom _XA_WIN_DESK;
 
 void
 InternUsefulAtoms (void)
 {
-  /* 
+  /*
    * Create priority colors if necessary.
    */
   _XA_MIT_PRIORITY_COLORS = XInternAtom (dpy, "_MIT_PRIORITY_COLORS", False);
@@ -556,9 +620,12 @@ InternUsefulAtoms (void)
   _XA_WM_PROTOCOLS = XInternAtom (dpy, "WM_PROTOCOLS", False);
   _XA_WM_TAKE_FOCUS = XInternAtom (dpy, "WM_TAKE_FOCUS", False);
   _XA_WM_DELETE_WINDOW = XInternAtom (dpy, "WM_DELETE_WINDOW", False);
-  _XA_WM_DESKTOP = XInternAtom (dpy, "WM_DESKTOP", False);
   _XA_MwmAtom = XInternAtom (dpy, "_MOTIF_WM_HINTS", False);
-  return;
+  _XA_WIN_STATE = XInternAtom (dpy, "_WIN_STATE", False);
+  _XROOTPMAP_ID = XInternAtom (dpy, "_XROOTPMAP_ID", False);
+  _AS_STYLE = XInternAtom (dpy, "_AS_STYLE", False);
+  _AS_MODULE_SOCKET = XInternAtom (dpy, "_AS_MODULE_SOCKET", False);
+  _XA_WIN_DESK = XInternAtom (dpy, "_WIN_DESK", False);
 }
 
 /***********************************************************************
@@ -800,8 +867,8 @@ InitVariables (int shallresetdesktop)
 
       Scr.CurrentDesk = 0;
 
-      if ((XGetWindowProperty (dpy, Scr.Root, _XA_WM_DESKTOP, 0L, 1L, True,
-			       _XA_WM_DESKTOP, &atype, &aformat, &nitems,
+      if ((XGetWindowProperty (dpy, Scr.Root, _XA_WIN_DESK, 0L, 1L, True,
+			       AnyPropertyType, &atype, &aformat, &nitems,
 			       &bytes_remain, &prop)) == Success)
 	{
 	  if (prop != NULL)
@@ -895,13 +962,24 @@ InitModifiers (void)
 void
 Reborder ()
 {
+  ASWindow *t;
+
   /* remove afterstep frame from all windows */
   XGrabServer (dpy);
 
   InstallWindowColormaps (&Scr.ASRoot);		/* force reinstall */
 
-  while (Scr.ASRoot.next != NULL)
-    Destroy (Scr.ASRoot.next, False);
+  for (t = &Scr.ASRoot; t->next != NULL; t = t->next);
+  while (t != &Scr.ASRoot)
+    {
+      ASWindow *tmp = t->prev;
+      Destroy (t, False);
+      t = tmp;
+    }
+#ifndef NO_TEXTURE
+  if (DecorateFrames)
+    frame_free_data (NULL, True);
+#endif /* !NO_TEXTURE */
 
   XUngrabServer (dpy);
   XSetInputFocus (dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
@@ -959,10 +1037,21 @@ Done (int restart, char *command)
   /* Close all my pipes */
   ClosePipes ();
 
+  /* freeing up memory */
+  InitASDirNames (True);
+  free_func_hash ();
+
+#ifdef HAVE_XINERAMA
+  if( Scr.xinerama_screens )
+  {
+	  free(Scr.xinerama_screens);
+	  Scr.xinerama_screens_num = 0;
+	  Scr.xinerama_screens = NULL;
+  }
+#endif /* XINERAMA */
+
   if (restart)
     {
-      SaveDesktopState ();	/* I wonder why ... */
-
       /* Really make sure that the connection is closed and cleared! */
       XSelectInput (dpy, Scr.Root, 0);
       XSync (dpy, 0);
@@ -1000,33 +1089,25 @@ Done (int restart, char *command)
     }
   else
     {
+      extern Atom _XA_WIN_DESK;
+      XDeleteProperty (dpy, Scr.Root, _XA_WIN_DESK);
+
 #ifndef NO_SAVEWINDOWS
       SaveWindowsOpened ();
 #endif
 
 #ifdef DEBUG_ALLOCS
       {				/* free up memory */
-	extern int *writePipes;
-	extern int *readPipes;
-	extern int *pipeOn;
-	extern unsigned long *PipeMask;
-	extern char **pipeName;
-	extern struct queue_buff_struct **pipeQueue;
 	extern char *global_base_file;
 
-	/* free display strings; can't do this in main(), because some OS's 
+	/* free display strings; can't do this in main(), because some OS's
 	 * don't copy the environment variables properly */
 	free (display_string);
 	free (rdisplay_string);
 
 	free (Scr.lock_mods);
 	/* module stuff */
-	free (writePipes);
-	free (readPipes);
-	free (pipeOn);
-	free (PipeMask);
-	free (pipeName);
-	free (pipeQueue);
+	module_init (1);
 	free (global_base_file);
 	/* configure stuff */
 	init_old_look_variables (True);
@@ -1036,6 +1117,25 @@ Done (int restart, char *command)
 	InitDatabase (True);
 	while (Scr.first_menu != NULL)
 	  DeleteMenuRoot (Scr.first_menu);
+	/* global drawing GCs */
+	if (Scr.NormalGC != NULL)
+	  XFreeGC (dpy, Scr.NormalGC);
+	if (Scr.StippleGC != NULL)
+	  XFreeGC (dpy, Scr.StippleGC);
+	if (Scr.DrawGC != NULL)
+	  XFreeGC (dpy, Scr.DrawGC);
+	if (Scr.LineGC != NULL)
+	  XFreeGC (dpy, Scr.LineGC);
+	if (Scr.ScratchGC1 != NULL)
+	  XFreeGC (dpy, Scr.ScratchGC1);
+	if (Scr.ScratchGC2 != NULL)
+	  XFreeGC (dpy, Scr.ScratchGC2);
+	if (Scr.gray_bitmap != None)
+	  XFreePixmap (dpy, Scr.gray_bitmap);
+	/* balloons */
+	balloon_init (1);
+	/* pixmap references */
+	pixmap_ref_purge ();
       }
       print_unfreed_mem ();
 #endif /*DEBUG_ALLOCS */
@@ -1071,7 +1171,7 @@ ASErrorHandler (Display * dpy, XErrorEvent * event)
 {
   extern int last_event_type;
 
-  /* some errors are acceptable, mostly they're caused by 
+  /* some errors are acceptable, mostly they're caused by
    * trying to update a lost  window */
   if ((event->error_code == BadWindow) || (event->request_code == X_GetGeometry) ||
       (event->error_code == BadDrawable) || (event->request_code == X_SetInputFocus) ||
@@ -1118,7 +1218,7 @@ usage (void)
 
 /***************************************************************************
  * checkPanFrames hides PanFrames if they are on the very border of the
- * VIRTUELL screen and EdgeWrap for that direction is off. 
+ * VIRTUELL screen and EdgeWrap for that direction is off.
  * (A special cursor for the EdgeWrap border could be nice) HEDU
  ****************************************************************************/
 void
@@ -1214,7 +1314,7 @@ raisePanFrames (void)
 
 /****************************************************************************
  *
- * Creates the windows for edge-scrolling 
+ * Creates the windows for edge-scrolling
  *
  ****************************************************************************/
 void
@@ -1270,28 +1370,3 @@ initPanFrames ()
 }
 
 #endif /* NO_VIRTUAL */
-
-/****************************************************************************
- *
- * Save Desktop State
- *
- ****************************************************************************/
-void
-SaveDesktopState ()
-{
-  ASWindow *t;
-  unsigned long data[1];
-
-  for (t = Scr.ASRoot.next; t != NULL; t = t->next)
-    {
-      data[0] = (unsigned long) t->Desk;
-      XChangeProperty (dpy, t->w, _XA_WM_DESKTOP, _XA_WM_DESKTOP, 32,
-		       PropModeReplace, (unsigned char *) data, 1);
-    }
-
-  data[0] = (unsigned long) Scr.CurrentDesk;
-  XChangeProperty (dpy, Scr.Root, _XA_WM_DESKTOP, _XA_WM_DESKTOP, 32,
-		   PropModeReplace, (unsigned char *) data, 1);
-
-  XSync (dpy, 0);
-}
