@@ -33,18 +33,26 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
 #include <X11/Xatom.h>
 #include <X11/Xproto.h>
+
+#include "../../include/aftersteplib.h"
 #include "../../include/afterstep.h"
-#include "../../include/misc.h"
-#include "../../include/screen.h"
 #include "../../include/parse.h"
-#include "../../include/menus.h"
+#include "../../include/misc.h"
+#include "../../include/style.h"
+#include "../../include/screen.h"
 #ifndef NO_TEXTURE
+#include "../../include/ascolor.h"
 #include "../../include/stepgfx.h"
 #endif
 
+#include "menus.h"
+
 extern Atom _XA_MwmAtom;
+extern Atom _XA_WIN_STATE;
 
 /* Motif WM window hints structure */
 typedef struct
@@ -90,6 +98,26 @@ MwmHints;
 #define PROP_MOTIF_WM_HINTS_ELEMENTS  5
 #define PROP_MWM_HINTS_ELEMENTS       PROP_MOTIF_WM_HINTS_ELEMENTS
 
+/* Gnome WM window hints */
+#define WIN_HINTS_SKIP_FOCUS      (1<<0)	/*"alt-tab" skips this win */
+#define WIN_HINTS_SKIP_WINLIST    (1<<1)	/*do not show in window list */
+#define WIN_HINTS_SKIP_TASKBAR    (1<<2)	/*do not show on taskbar */
+#define WIN_HINTS_GROUP_TRANSIENT (1<<3)	/*Reserved - definition is unclear */
+#define WIN_HINTS_FOCUS_ON_CLICK  (1<<4)	/*app only accepts focus if clicked */
+#define WIN_HINTS_DO_NOT_COVER    (1<<5)	/*attempt to not cover this window */
+
+/* Gnome WM window states */
+#define WIN_STATE_STICKY          (1<<0)	/*everyone knows sticky */
+#define WIN_STATE_MINIMIZED       (1<<1)	/*Reserved - definition is unclear */
+#define WIN_STATE_MAXIMIZED_VERT  (1<<2)	/*window in maximized V state */
+#define WIN_STATE_MAXIMIZED_HORIZ (1<<3)	/*window in maximized H state */
+#define WIN_STATE_HIDDEN          (1<<4)	/*not on taskbar but window visible */
+#define WIN_STATE_SHADED          (1<<5)	/*shaded (MacOS / Afterstep style) */
+#define WIN_STATE_HID_WORKSPACE   (1<<6)	/*not on current desktop */
+#define WIN_STATE_HID_TRANSIENT   (1<<7)	/*owner of transient is hidden */
+#define WIN_STATE_FIXED_POSITION  (1<<8)	/*window is fixed in position even */
+#define WIN_STATE_ARRANGE_IGNORE  (1<<9)	/*ignore for auto arranging */
+
 extern ASWindow *Tmp_win;
 
 /****************************************************************************
@@ -111,25 +139,61 @@ GetMwmHints (ASWindow * t)
 			(unsigned char **) &mwm_hints) == Success))
     {
       if (nitems >= 4)
-	{
-#if 0
-	  {
-	    int i;
-	    FILE *fp = fopen ("/tmp/mwmhints", "a");
-	    fprintf (fp, "%30s ", t->name);
-	    for (i = 0; i < nitems; i++)
-	      fprintf (fp, "%08x ", ((int *) mwm_hints)[i]);
-	    fprintf (fp, "\n");
-	    fclose (fp);
-	  }
-#endif
-	  t->mwm_hints = mwm_hints;
-	}
+	t->mwm_hints = mwm_hints;
     }
   else
     t->mwm_hints = NULL;
 }
 
+unsigned long
+GetGnomeState (ASWindow * t)
+{
+  int actual_format;
+  Atom actual_type;
+  unsigned long nitems, bytesafter;
+  unsigned long state = 0;
+  unsigned long *data;
+
+  if (XGetWindowProperty (dpy, t->w, _XA_WIN_STATE, 0, 1, False, XA_CARDINAL,
+			  &actual_type, &actual_format, &nitems, &bytesafter,
+			  (unsigned char **) &data) == Success && data)
+    {
+      state = *data;
+      XFree (data);
+    }
+
+  return state;
+}
+
+unsigned long
+SetGnomeState (ASWindow * t)
+{
+  unsigned long state = GetGnomeState (t);
+
+  state &= ~(WIN_STATE_STICKY | WIN_STATE_MINIMIZED |
+	     WIN_STATE_MAXIMIZED_HORIZ | WIN_STATE_MAXIMIZED_VERT |
+	     WIN_STATE_SHADED);
+  if (t->flags & STICKY)
+    state |= WIN_STATE_STICKY;
+  if (t->flags & ICONIFIED)
+    state |= WIN_STATE_MINIMIZED;
+  if (t->flags & MAXIMIZED)
+    {
+      if (t->orig_x != t->frame_x || t->orig_wd != t->frame_width)
+	state |= WIN_STATE_MAXIMIZED_HORIZ;
+      if (t->orig_y != t->frame_y || t->orig_ht != t->frame_height)
+	state |= WIN_STATE_MAXIMIZED_VERT;
+    }
+/*if (t->flags & WINDOWLISTSKIP) state |= WIN_STATE_HIDDEN; */
+  if (t->flags & SHADED)
+    state |= WIN_STATE_SHADED;
+
+  XChangeProperty (dpy, t->w, _XA_WIN_STATE, XA_CARDINAL, 32, PropModeReplace,
+		   (unsigned char *) &state, 1);
+
+  return state;
+}
+
 Bool
 is_function_bound_to_button (int button, int function)
 {
@@ -240,7 +304,7 @@ SelectDecor (ASWindow * t, unsigned long tflags, int border_width,
       || ((!(Scr.flags & DecorateTransients)) && (t->flags & TRANSIENT)))
     decor &= ~MWM_DECOR_TITLE;
 
-  if ((tflags & NOBORDER_FLAG)
+  if ((tflags & NOHANDLES_FLAG)
       || ((!(Scr.flags & DecorateTransients)) && (t->flags & TRANSIENT)))
     decor &= ~MWM_DECOR_RESIZEH;
 
@@ -249,13 +313,14 @@ SelectDecor (ASWindow * t, unsigned long tflags, int border_width,
 
 #ifdef SHAPE
   if (t->wShaped)
-    decor &= ~(BORDER | MWM_DECOR_RESIZEH);
+    decor &= ~(BORDER | MWM_DECOR_RESIZEH | FRAME);
 #endif
   /* Assume no decorations, and build up */
-  t->flags &= ~(BORDER | TITLE);
+  t->flags &= ~(BORDER | TITLE | FRAME);
   t->boundary_width = 0;
   t->boundary_height = 0;
   t->corner_width = 0;
+  t->title_width = 0;
   t->title_height = 0;
   t->button_height = 0;
 
@@ -263,12 +328,16 @@ SelectDecor (ASWindow * t, unsigned long tflags, int border_width,
     {
       /* a titlebar with no buttons in it */
       t->flags |= TITLE;
-      t->title_height = Scr.TitleHeight;
     }
   if (decor & MWM_DECOR_RESIZEH)
     {
       /* a wide border, with corner tiles */
-      t->flags |= BORDER;
+#ifndef NO_TEXTURE
+      if (DecorateFrames)
+	t->flags |= FRAME;
+      else
+#endif /* !NO_TEXTURE */
+	t->flags |= BORDER;
     }
   if (!(decor & MWM_DECOR_MENU))
     disable_titlebuttons_with_function (t, F_POPUP);
@@ -279,10 +348,14 @@ SelectDecor (ASWindow * t, unsigned long tflags, int border_width,
 
   t->boundary_width = 0;
   t->boundary_height = (t->flags & BORDER) ? Scr.BoundaryWidth : 0;
-  t->corner_width = Scr.TitleHeight + t->boundary_height;
+  t->corner_width = 16 + t->boundary_height;
 
-  t->bw = 1;
-  if (tflags & NOBORDER_FLAG)
+  if (t->flags & FRAME)
+    t->bw = 0;
+  else
+    t->bw = 1;
+
+  if (tflags & NOHANDLES_FLAG)
     t->bw = border_width;
   if (t->boundary_height == 0)
     t->flags &= ~BORDER;
@@ -326,7 +399,7 @@ check_allowed_function (MenuItem * mi)
     return 0;
 
   if ((mi->func == F_SHADE) && (Tmp_win) &&
-      (!(Tmp_win->functions & MWM_FUNC_MAXIMIZE)))
+   (!(Tmp_win->functions & MWM_FUNC_MAXIMIZE) || !(Tmp_win->flags & TITLE)))
     return 0;
 
   if ((mi->func == F_DELETE) && (Tmp_win) &&
@@ -344,7 +417,7 @@ check_allowed_function (MenuItem * mi)
   if (mi->func == F_NOP)
     return 0;
 
-  if (mi->func == F_FUNCTION)
+  if (mi->func == F_FUNCTION && mi->item != NULL)
     {
       /* Hard part! What to do now? */
       /* Hate to do it, but for lack of a better idea,
@@ -426,7 +499,7 @@ check_allowed_function2 (int function, ASWindow * t)
     return 0;
 
   if ((function == F_SHADE) && (t) &&
-      (!(t->functions & MWM_FUNC_MAXIMIZE)))
+      (!(t->functions & MWM_FUNC_MAXIMIZE) || !(t->flags & TITLE)))
     return 0;
 
   if ((function == F_DELETE) && (t) &&
