@@ -39,22 +39,26 @@
 #include <sys/select.h>
 #endif
 
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
 #include <X11/Xatom.h>
 
+#include "../../include/aftersteplib.h"
 #include "../../include/afterstep.h"
 #include "../../include/module.h"
-#include "../../include/menus.h"
-#include "../../include/misc.h"
 #include "../../include/parse.h"
+#include "../../include/misc.h"
+#include "../../include/style.h"
 #include "../../include/screen.h"
+#include "../../include/loadimg.h"
 
 #ifdef SHAPE
 #include <X11/extensions/shape.h>
 #endif /* SHAPE */
 
-extern int menuFromFrameOrWindowOrTitlebar;
+#include "menus.h"
 
-static void ChangeIcon ();
+extern int menuFromFrameOrWindowOrTitlebar;
 
 int Context = C_NO_CONTEXT;	/* current button press context */
 int Button = 0;
@@ -81,7 +85,7 @@ warp_grab (ASWindow * t)
 {
   XWindowAttributes attributes;
 
-  /* we're watching all key presses and accept mouse cursor motion events 
+  /* we're watching all key presses and accept mouse cursor motion events
      so we will be able to tell when warp mode is finished */
   XGetWindowAttributes (dpy, t->frame, &attributes);
   XSelectInput (dpy, t->frame, attributes.your_event_mask | (PointerMotionMask | KeyPressMask));
@@ -133,6 +137,9 @@ DispatchEvent ()
   last_event_type = Event.type;
   last_event_window = w;
 
+  /* handle balloon events specially */
+  balloon_handle_event (&Event);
+
   /* handle menu events specially */
   if (HandleMenuEvent (NULL, &Event) == True)
     return;
@@ -155,6 +162,8 @@ DispatchEvent ()
       HandleUnmapNotify ();
       break;
     case ButtonPress:
+      /* if warping, a button press, non-warp keypress, or pointer motion
+       * indicates that the warp is done */
       if ((Tmp_win != NULL) && (warp_in_process))
 	warp_ungrab (Tmp_win, True);
       HandleButtonPress ();
@@ -164,10 +173,12 @@ DispatchEvent ()
       break;
     case LeaveNotify:
       HandleLeaveNotify ();
-      /* if we're warping, then we must return to accept only normal events again
-         in that window we left */
+#if 0
+      /* if warping, leaving a window means that we need to ungrab, but
+       * the ungrab should be taken care of by the FocusOut */
       if ((warp_in_process) && (Tmp_win != NULL))
 	warp_ungrab (Tmp_win, False);
+#endif
       break;
     case FocusIn:
       HandleFocusIn ();
@@ -179,7 +190,15 @@ DispatchEvent ()
 	    ChangeWarpIndex (Tmp_win->warp_index, F_WARP_F);
 	}
       break;
+    case FocusOut:
+      /* if warping, this is the normal way to determine that we should ungrab
+       * window events */
+      if (Tmp_win != NULL && warp_in_process)
+	warp_ungrab (Tmp_win, False);
+      break;
     case MotionNotify:
+      /* if warping, a button press, non-warp keypress, or pointer motion
+       * indicates that the warp is done */
       if ((warp_in_process) && (Tmp_win != NULL))
 	warp_ungrab (Tmp_win, True);
       break;
@@ -221,13 +240,15 @@ DispatchEvent ()
 void
 HandleEvents ()
 {
-  while (TRUE)
+  while (True)
     {
       last_event_type = 0;
-      if (My_XNextEvent (dpy, &Event))
-	{
-	  DispatchEvent ();
-	}
+      if (AS_XNextEvent (dpy, &Event))
+	  {
+/*fprintf( stderr, "%s:%d Received event %d\n", __FUNCTION__, __LINE__, Event.type );*/
+		DispatchEvent ();
+
+	  }
     }
 }
 
@@ -242,7 +263,7 @@ GetContext (ASWindow * t, XEvent * e, Window * w)
 {
   int i;
 
-  if (t == NULL)
+  if (t == NULL || e->xany.window == None)
     return C_ROOT;
 
   *w = e->xany.window;
@@ -264,7 +285,9 @@ GetContext (ASWindow * t, XEvent * e, Window * w)
 	{
 	  if (e->xbutton.subwindow != None)
 	    *w = e->xany.window = e->xbutton.subwindow;
-	  else if ((*w == t->frame) && (e->xbutton.y < Scr.TitleHeight + 2))
+	  else if (*w == t->frame &&
+		   e->xbutton.x >= t->title_x && e->xbutton.x < t->title_x + t->title_width + 2 * t->bw &&
+		   e->xbutton.y >= t->title_y && e->xbutton.y < t->title_y + t->title_height + 2 * t->bw)
 	    return C_TITLE;
 	}
     }
@@ -285,9 +308,6 @@ GetContext (ASWindow * t, XEvent * e, Window * w)
 		return C_L1 << i;
 	      }
 	  }
-    }
-  if (*w == t->title_w)
-    {
       for (i = 0; i < Scr.nr_right_buttons; i++)
 	if (t->right_w[i] != None)
 	  {
@@ -305,15 +325,18 @@ GetContext (ASWindow * t, XEvent * e, Window * w)
     }
   if (*w == t->title_w)
     return C_TITLE;
-#ifdef NO_ICON_BACKGROUND
-  if (*w == t->icon_w)
-    Context = C_ICON;
-#endif
+  if (*w == t->icon_title_w)
+    return C_ICON;
   if ((*w == t->icon_pixmap_w) || (t->flags & ICONIFIED))
     return C_ICON;
   if ((*w == t->frame) || (*w == t->side))
     return C_SIDEBAR;
 
+  for (i = 0; i < 8; i++)
+    {
+      if (*w == t->fw[i])
+	return C_FRAME;
+    }
   for (i = 0; i < 2; i++)
     {
       if (*w == t->corners[i])
@@ -367,13 +390,12 @@ HandleFocusIn ()
   if (!Tmp_win)
     {
       SetBorder (Scr.Hilite, False, True, True, None);
-      Broadcast (M_FOCUS_CHANGE, 3, 0, 0, 0, 0, 0, 0, 0);
+      Broadcast (M_FOCUS_CHANGE, 3, 0L, 0L, 0L);
     }
   else if (Tmp_win != Scr.Hilite)
     {
       SetBorder (Tmp_win, True, True, True, None);
-      Broadcast (M_FOCUS_CHANGE, 3, Tmp_win->w,
-		 Tmp_win->frame, (unsigned long) Tmp_win, 0, 0, 0, 0);
+      Broadcast (M_FOCUS_CHANGE, 3, Tmp_win->w, Tmp_win->frame, (unsigned long) Tmp_win);
     }
 }
 
@@ -393,7 +415,7 @@ HandleKeyPress ()
   Context = GetContext (Tmp_win, &Event, &dummy);
 
   modifier = (Event.xkey.state & Scr.nonlock_mods);
-  for (key = Scr.FuncKeyRoot.next; key != NULL; key = key->next)
+  for (key = Scr.FuncKeyRoot; key != NULL; key = key->next)
     {
       ButtonWindow = Tmp_win;
       /* Here's a real hack - some systems have two keys with the
@@ -406,8 +428,9 @@ HandleKeyPress ()
 	   (key->mods == AnyModifier)) &&
 	  (key->cont & Context))
 	{
+	  extern int AutoReverse;
 	  /* check if the warp key was pressed */
-	  warp_in_process = ((key->func == F_WARP_B) || (key->func == F_WARP_F));
+	  warp_in_process = ((key->func == F_WARP_B || key->func == F_WARP_F) && AutoReverse == 2);
 	  if (warp_in_process)
 	    warping_direction = key->func;
 
@@ -467,6 +490,23 @@ HandlePropertyNotify ()
   int num;
 #endif
 
+  /* force updates for "transparent" windows */
+  if (Event.xproperty.atom == _XROOTPMAP_ID && Event.xproperty.window == Scr.Root)
+    {
+      ASWindow *win;
+      for (win = Scr.ASRoot.next; win != NULL; win = win->next)
+	if (SetTransparency (win))
+	  SetBorder (win, Scr.Hilite == win, True, True, None);
+      /* use move_menu() to update transparent menus; this is a kludge, but it works */
+      if ((*Scr.MSMenuTitle).texture_type == 129 || (*Scr.MSMenuItem).texture_type == 129 || (*Scr.MSMenuHilite).texture_type == 129)
+	{
+	  MenuRoot *menu;
+	  for (menu = Scr.first_menu; menu != NULL; menu = menu->next)
+	    if ((*menu).is_mapped)
+	      move_menu (menu, (*menu).x, (*menu).y);
+	}
+    }
+
   if ((!Tmp_win) || (XGetGeometry (dpy, Tmp_win->w, &JunkRoot, &JunkX, &JunkY,
 			&JunkWidth, &JunkHeight, &JunkBW, &JunkDepth) == 0))
     return;
@@ -518,24 +558,26 @@ HandlePropertyNotify ()
       BroadcastName (M_WINDOW_NAME, Tmp_win->w, Tmp_win->frame,
 		     (unsigned long) Tmp_win, Tmp_win->name);
 
+	  if( Scr.flags& FollowTitleChanges )
+	      ChangeIcon(Tmp_win);
+          
       /* fix the name in the title bar */
       if (!(Tmp_win->flags & ICONIFIED))
 	SetTitleBar (Tmp_win, (Scr.Hilite == Tmp_win), True);
       else
-	ChangeIcon ();
+	DrawIconWindow (Tmp_win);
 
 /*
  * if the icon name is NoName, set the name of the icon to be
- * the same as the window 
+ * the same as the window
  */
       if (Tmp_win->icon_name == NoName)
 	{
 	  Tmp_win->icon_name = Tmp_win->name;
 	  BroadcastName (M_ICON_NAME, Tmp_win->w, Tmp_win->frame,
 			 (unsigned long) Tmp_win, Tmp_win->icon_name);
-#ifdef NO_ICON_BACKGROUND
-	  RedoIconName (Tmp_win);
-#endif
+	  if (Textures.flags & SeparateButtonTitle)
+	    RedoIconName (Tmp_win);
 	}
       break;
 
@@ -583,10 +625,9 @@ HandlePropertyNotify ()
 		     (unsigned long) Tmp_win, Tmp_win->icon_name);
       if ((Tmp_win->flags & ICONIFIED))
 	{
-	  ChangeIcon ();
-#ifdef NO_ICON_BACKGROUND
-	  RedoIconName (Tmp_win);
-#endif
+	  DrawIconWindow (Tmp_win);
+	  if (Textures.flags & SeparateButtonTitle)
+	    RedoIconName (Tmp_win);
 	}
       break;
 
@@ -599,67 +640,9 @@ HandlePropertyNotify ()
 	return;
 
       if ((Tmp_win->wmhints->flags & IconPixmapHint) ||
-	  (Tmp_win->wmhints->flags & IconWindowHint))
-	{
-
-	  if (Tmp_win->icon_bitmap_file == Scr.DefaultIcon)
-	    Tmp_win->icon_bitmap_file = (char *) 0;
-
-	  if (!(Tmp_win->flags & SUPPRESSICON))
-	    {
-#ifdef NO_ICON_BACKGROUND
-	      if (Tmp_win->icon_w)
-		XDestroyWindow (dpy, Tmp_win->icon_w);
-	      XDeleteContext (dpy, Tmp_win->icon_w, ASContext);
-#endif
-	      if (Tmp_win->flags & ICON_OURS)
-		{
-		  if (Tmp_win->icon_pixmap_w != None)
-		    {
-		      XDestroyWindow (dpy, Tmp_win->icon_pixmap_w);
-		      XDeleteContext (dpy, Tmp_win->icon_pixmap_w, ASContext);
-		    }
-		}
-	      else
-		XUnmapWindow (dpy, Tmp_win->icon_pixmap_w);
-	    }
-#ifdef NO_ICON_BACKGROUND
-	  Tmp_win->icon_w = None;
-#endif
-	  Tmp_win->icon_pixmap_w = None;
-	  Tmp_win->iconPixmap = (Window) NULL;
-	  if (Tmp_win->flags & ICONIFIED)
-	    {
-	      Tmp_win->flags &= ~ICONIFIED;
-	      Tmp_win->flags &= ~ICON_UNMAPPED;
-	      CreateIconWindow (Tmp_win, Tmp_win->icon_x_loc, Tmp_win->icon_y_loc);
-	      Broadcast (M_ICONIFY, 7, Tmp_win->w, Tmp_win->frame,
-			 (unsigned long) Tmp_win,
-			 Tmp_win->icon_x_loc,
-			 Tmp_win->icon_y_loc,
-			 Tmp_win->icon_p_width,
-			 Tmp_win->icon_p_height);
-	      BroadcastConfig (M_CONFIGURE_WINDOW, Tmp_win);
-
-	      if (!(Tmp_win->flags & SUPPRESSICON))
-		{
-		  LowerWindow (Tmp_win);
-		  AutoPlace (Tmp_win);
-		  if (Tmp_win->Desk == Scr.CurrentDesk)
-		    {
-#ifdef NO_ICON_BACKGROUND
-		      if ((Tmp_win->icon_w != None) && (Scr.flags & IconTitle))
-			XMapWindow (dpy, Tmp_win->icon_w);
-#endif
-
-		      if (Tmp_win->icon_pixmap_w != None)
-			XMapWindow (dpy, Tmp_win->icon_pixmap_w);
-		    }
-		  DrawIconWindow (Tmp_win);
-		}
-	      Tmp_win->flags |= ICONIFIED;
-	    }
-	}
+	  (Tmp_win->wmhints->flags & IconWindowHint) ||
+	  !(Tmp_win->flags & (ICON_OURS | PIXMAP_OURS)))
+	ChangeIcon (Tmp_win);
       break;
 
     case XA_WM_NORMAL_HINTS:
@@ -737,16 +720,45 @@ HandleExpose ()
 
   if (Tmp_win)
     {
-      if ((Event.xany.window == Tmp_win->title_w))
+      Window w = Event.xany.window;
+      int i, title_only = 1;
+
+      flush_expose (Tmp_win->frame);
+      flush_expose (Tmp_win->Parent);
+
+      /* discard exposures of the frame or Parent windows */
+      if (Event.xexpose.window == Tmp_win->Parent)
+	return;
+      if (Event.xexpose.window == Tmp_win->frame)
 	{
-	  SetTitleBar (Tmp_win, (Scr.Hilite == Tmp_win), False);
+	  /*  if (Tmp_win->flags & FRAME && (!(Tmp_win->flags & SHADED)))
+	     frame_draw_frame (Tmp_win); */
+	  return;
 	}
+      flush_expose (Tmp_win->title_w);
+      for (i = 0; i < Scr.nr_left_buttons; i++)
+	if (Tmp_win->left_w[i] != None)
+	  flush_expose (Tmp_win->left_w[i]);
+      for (i = 0; i < Scr.nr_right_buttons; i++)
+	if (Tmp_win->right_w[i] != None)
+	  flush_expose (Tmp_win->right_w[i]);
+
+      if (flush_expose (Tmp_win->side) || w == Tmp_win->side)
+	title_only = 0;
+      if (flush_expose (Tmp_win->corners[0]) || w == Tmp_win->corners[0])
+	title_only = 0;
+      if (flush_expose (Tmp_win->corners[1]) || w == Tmp_win->corners[1])
+	title_only = 0;
+      if (flush_expose (Tmp_win->icon_pixmap_w) || w == Tmp_win->icon_pixmap_w)
+	title_only = 0;
+      if (flush_expose (Tmp_win->icon_title_w) || w == Tmp_win->icon_title_w)
+	title_only = 0;
+
+      if (title_only)
+	SetTitleBar (Tmp_win, (Scr.Hilite == Tmp_win), False);
       else
-	{
-	  SetBorder (Tmp_win, (Scr.Hilite == Tmp_win), True, True, Event.xany.window);
-	}
+	SetBorder (Tmp_win, (Scr.Hilite == Tmp_win), True, True, Event.xany.window);
     }
-  return;
 }
 
 
@@ -839,7 +851,7 @@ HandleMapRequest ()
 	  break;
 
 	case IconicState:
-	  Iconify (Tmp_win, 0, 0);
+	  Iconify (Tmp_win);
 	  break;
 
 	case ZoomState:
@@ -877,11 +889,6 @@ HandleMapNotify ()
    * when it really isn't.
    */
   XGrabServer (dpy);
-/* #ifdef NO_ICON_BACKGROUND
-   if (Tmp_win->icon_w)
-   XUnmapWindow(dpy, Tmp_win->icon_w);
-   #endif
- */
   if (Tmp_win->icon_pixmap_w != None)
     XUnmapWindow (dpy, Tmp_win->icon_pixmap_w);
   XMapSubwindows (dpy, Tmp_win->frame);
@@ -891,21 +898,16 @@ HandleMapNotify ()
       XMapWindow (dpy, Tmp_win->frame);
     }
   if (Tmp_win->flags & ICONIFIED)
-    Broadcast (M_DEICONIFY, 3, Tmp_win->w, Tmp_win->frame,
-	       (unsigned long) Tmp_win, 0, 0, 0, 0);
+    Broadcast (M_DEICONIFY, 3, Tmp_win->w, Tmp_win->frame, (unsigned long) Tmp_win);
   else
-    {
-      Broadcast (M_MAP, 3, Tmp_win->w, Tmp_win->frame,
-		 (unsigned long) Tmp_win, 0, 0, 0, 0);
-    }
+    Broadcast (M_MAP, 3, Tmp_win->w, Tmp_win->frame, (unsigned long) Tmp_win);
 
   if (Scr.flags & ClickToFocus)
     SetFocus (Tmp_win->w, Tmp_win, False);
 
   if ((!(Tmp_win->flags & (BORDER | TITLE))) && (Tmp_win->boundary_width < 2))
-    {
-      SetBorder (Tmp_win, False, True, True, Tmp_win->frame);
-    }
+    SetBorder (Tmp_win, False, True, True, Tmp_win->frame);
+
   XSync (dpy, 0);
   XUngrabServer (dpy);
   XFlush (dpy);
@@ -1001,8 +1003,8 @@ HandleUnmapNotify ()
    * The program may have unmapped the client window, from either
    * NormalState or IconicState.  Handle the transition to WithdrawnState.
    *
-   * We need to reparent the window back to the root (so that afterstep exiting 
-   * won't cause it to get mapped) and then throw away all state (pretend 
+   * We need to reparent the window back to the root (so that afterstep exiting
+   * won't cause it to get mapped) and then throw away all state (pretend
    * that we've received a DestroyNotify).
    */
   if (XTranslateCoordinates (dpy, Event.xunmap.window, Scr.Root,
@@ -1056,9 +1058,9 @@ HandleButtonPress ()
 {
   unsigned int modifier;
   MouseButton *MouseEntry;
-  char *Action;
   Window xPressedW;
   int LocalContext;
+  Bool AShandled = False;
 
   Context = GetContext (Tmp_win, &Event, &PressedW);
   LocalContext = Context;
@@ -1153,17 +1155,29 @@ HandleButtonPress ()
 	  /* got a match, now process it */
 	  if (MouseEntry->func != (int) NULL)
 	    {
-	      Action = MouseEntry->item ? MouseEntry->item->action : NULL;
-	      ExecuteFunction (MouseEntry->func, Action, Event.xany.window,
+	      ExecuteFunction (MouseEntry->func, MouseEntry->action,
+			       Event.xany.window,
 			       Tmp_win, &Event, Context, MouseEntry->val1,
 			       MouseEntry->val2,
 			       MouseEntry->val1_unit, MouseEntry->val2_unit,
 			       MouseEntry->menu, -1);
+	      AShandled = True;
 	      break;
 	    }
 	}
       MouseEntry = MouseEntry->NextButton;
     }
+
+  /* GNOME this click hasn't been taken by AfterStep */
+  if (!AShandled && Event.xany.window == Scr.Root)
+    {
+      extern Window GnomeProxyWin;
+
+      if (Event.type == ButtonPress)
+	XUngrabPointer (dpy, CurrentTime);
+      XSendEvent (dpy, GnomeProxyWin, False, SubstructureNotifyMask, &Event);
+    }
+
   PressedW = None;
 
   if (LocalContext != C_TITLE)
@@ -1189,6 +1203,7 @@ HandleEnterNotify ()
   /* look for a matching leaveNotify which would nullify this enterNotify */
   if (XCheckTypedWindowEvent (dpy, ewp->window, LeaveNotify, &d))
     {
+      balloon_handle_event (&d);
       StashEventTime (&d);
       if ((d.xcrossing.mode == NotifyNormal) &&
 	  (d.xcrossing.detail != NotifyInferior))
@@ -1203,7 +1218,7 @@ HandleEnterNotify ()
     {
       int delta_x = 0, delta_y = 0;
       /* this was in the HandleMotionNotify before, HEDU */
-      HandlePaging (Scr.EdgeScrollX, Scr.EdgeScrollY,
+      HandlePaging (NULL, Scr.EdgeScrollX, Scr.EdgeScrollY,
 		    &Event.xcrossing.x_root, &Event.xcrossing.y_root,
 		    &delta_x, &delta_y, True);
       return;
@@ -1241,7 +1256,6 @@ HandleEnterNotify ()
       else
 	InstallWindowColormaps (NULL);
     }
-  return;
 }
 
 
@@ -1306,54 +1320,17 @@ HandleConfigureRequest ()
    * to configuration requests for windows which have never been mapped.
    */
 
-#ifdef NO_ICON_BACKGROUND
-  if (!Tmp_win || (Tmp_win->icon_w == cre->window))
+  if (Tmp_win == NULL)
     {
-#else
-  if (!Tmp_win)
-    {
-#endif
       xwcm = cre->value_mask &
 	(CWX | CWY | CWWidth | CWHeight | CWBorderWidth);
       xwc.x = cre->x;
       xwc.y = cre->y;
 
-#ifdef NO_ICON_BACKGROUND
-      if ((Tmp_win) && ((Tmp_win->icon_w == cre->window)))
-	{
-	  Tmp_win->icon_xl_loc = cre->x;
-	  Tmp_win->icon_x_loc = cre->x +
-	    (Tmp_win->icon_w_width - Tmp_win->icon_p_width) / 2;
-	  Tmp_win->icon_y_loc = cre->y - Tmp_win->icon_p_height;
-	  if (!(Tmp_win->flags & ICON_UNMAPPED))
-	    Broadcast (M_ICON_LOCATION, 7, Tmp_win->w, Tmp_win->frame,
-		       (unsigned long) Tmp_win,
-		       Tmp_win->icon_x_loc, Tmp_win->icon_y_loc,
-		       Tmp_win->icon_w_width,
-		       Tmp_win->icon_w_height + Tmp_win->icon_p_height);
-	}
-#endif
-
       xwc.width = cre->width;
       xwc.height = cre->height;
       xwc.border_width = cre->border_width;
       XConfigureWindow (dpy, Event.xany.window, xwcm, &xwc);
-
-      if (Tmp_win)
-	{
-	  xwc.x = Tmp_win->icon_x_loc;
-	  xwc.y = Tmp_win->icon_y_loc - Tmp_win->icon_p_height;
-	  xwcm = cre->value_mask & (CWX | CWY);
-	  if (Tmp_win->icon_pixmap_w != None)
-	    XConfigureWindow (dpy, Tmp_win->icon_pixmap_w, xwcm, &xwc);
-	  xwc.x = Tmp_win->icon_x_loc;
-	  xwc.y = Tmp_win->icon_y_loc;
-	  xwcm = cre->value_mask & (CWX | CWY);
-#ifdef NO_ICON_BACKGROUND
-	  if (Tmp_win->icon_w != None)
-	    XConfigureWindow (dpy, Tmp_win->icon_w, xwcm, &xwc);
-#endif
-	}
       return;
     }
   if (cre->value_mask & CWStackMode)
@@ -1382,32 +1359,26 @@ HandleConfigureRequest ()
   }
 #endif /* SHAPE */
 
-  /* Don't modify frame_XXX fields before calling SetupWindow! */
-  x = Tmp_win->frame_x;
-  y = Tmp_win->frame_y;
-  width = Tmp_win->frame_width;
-  height = Tmp_win->frame_height;
-
   /* for restoring */
   if (cre->value_mask & CWBorderWidth)
     {
       Tmp_win->old_bw = cre->border_width;
     }
-  /* override even if border change */
 
-  if (cre->value_mask & CWX)
-    x = cre->x - Tmp_win->boundary_width - Tmp_win->bw;
-  if (cre->value_mask & CWY)
-    y = cre->y - (Tmp_win->title_height + Tmp_win->bw) - Tmp_win->bw;
-  if (cre->value_mask & CWWidth)
-    width = cre->width + 2 * Tmp_win->boundary_width;
-  if (cre->value_mask & CWHeight)
-    height = cre->height + (Tmp_win->title_height + Tmp_win->bw) + Tmp_win->boundary_height;
+  x = Tmp_win->frame_x;
+  y = Tmp_win->frame_y;
+  width = Tmp_win->frame_width;
+  height = Tmp_win->frame_height;
+  get_resize_geometry (Tmp_win, cre->x, cre->y, cre->width, cre->height,
+		       (cre->value_mask & CWX) ? &x : NULL,
+		       (cre->value_mask & CWY) ? &y : NULL,
+		       (cre->value_mask & CWWidth) ? &width : NULL,
+		       (cre->value_mask & CWHeight) ? &height : NULL);
 
   /*
    * SetupWindow (x,y) are the location of the upper-left outer corner and
    * are passed directly to XMoveResizeWindow (frame).  The (width,height)
-   * are the inner size of the frame.  The inner width is the same as the 
+   * are the inner size of the frame.  The inner width is the same as the
    * requested client window width; the inner height is the same as the
    * requested client window height plus any title bar slop.
    */
@@ -1437,8 +1408,9 @@ HandleShapeNotify (void)
 }
 #endif /* SHAPE */
 
+#if 1				/* see SetTimer() */
 /**************************************************************************
- * 
+ *
  * For auto-raising windows, this routine is called
  *
  *************************************************************************/
@@ -1449,7 +1421,7 @@ enterAlarm (int nonsense)
   alarmed = True;
   signal (SIGALRM, enterAlarm);
 }
-
+#endif /* 1 */
 
 /***************************************************************************
  *
@@ -1457,176 +1429,120 @@ enterAlarm (int nonsense)
  *
  ****************************************************************************/
 int
-My_XNextEvent (Display * dpy, XEvent * event)
+AS_XNextEvent (Display * dpy, XEvent * event)
 {
   extern int fd_width, x_fd;
-  struct itimerval value;
   fd_set in_fdset, out_fdset;
-  Window child;
-  Window targetWindow;
-  int i, count;
+  int i;
   int retval;
+  struct timeval tv;
+  struct timeval *t = NULL;
+  extern module_t *Module;
+  int max_fd = 0 ;
 
-  /* Do this prior to the select() call, in case the timer already expired,
-   * in which case the select would never return. */
-  if (alarmed)
+  XFlush (dpy);
+  if (XPending (dpy))
     {
-      alarmed = False;
-      XQueryPointer (dpy, Scr.Root, &JunkRoot, &child, &JunkX, &JunkY, &JunkX,
-		     &JunkY, &JunkMask);
-      if ((Scr.Focus != NULL) && (child == Scr.Focus->frame))
-	{
-	  if (!(Scr.Focus->flags & VISIBLE))
-	    {
-	      RaiseWindow (Scr.Focus);
-	    }
-	}
-      return 0;
+	  XNextEvent (dpy, event);
+	  StashEventTime (event);
+	  return 1;
     }
-#ifndef TIME_WITH_SYS_TIME
-  value.it_value.tv_usec = 0;
-  value.it_value.tv_sec = 0;
-#else
-  getitimer (ITIMER_REAL, &value);
-#endif
 
   FD_ZERO (&in_fdset);
   FD_SET (x_fd, &in_fdset);
+  max_fd = x_fd ;
   FD_ZERO (&out_fdset);
-  for (i = 0; i < npipes; i++)
-    {
-      if (readPipes[i] >= 0)
-	{
-	  FD_SET (readPipes[i], &in_fdset);
-	}
-    }
 
+  if( module_fd >= 0 )
+  {
+  	FD_SET (module_fd, &in_fdset);
+	if( max_fd < module_fd ) max_fd = module_fd ;
+  }
   for (i = 0; i < npipes; i++)
-    {
-      if (pipeQueue[i] != NULL)
-	{
-	  FD_SET (writePipes[i], &out_fdset);
-	}
-    }
+	  if (Module[i].fd >= 0)
+	  {
+	  	FD_SET (Module[i].fd, &in_fdset);
+		if( max_fd < Module[i].fd ) max_fd = Module[i].fd ;
+	  	if (Module[i].output_queue != NULL)
+	  		FD_SET (Module[i].fd, &out_fdset);
+	  }
+  /* watch for timeouts */
+  if (timer_delay_till_next_alarm ((time_t *) & tv.tv_sec, (time_t *) & tv.tv_usec))
+    t = &tv;
 
+#if 1				/* see SetTimer() */
+  {
+    struct itimerval value;
+    Window child;
+    /* Do this prior to the select() call, in case the timer already expired,
+     * in which case the select would never return. */
+    if (alarmed)
+      {
+	alarmed = False;
+	XQueryPointer (dpy, Scr.Root, &JunkRoot, &child, &JunkX, &JunkY, &JunkX,
+		       &JunkY, &JunkMask);
+	if ((Scr.Focus != NULL) && (child == Scr.Focus->frame))
+	  {
+	    if (!(Scr.Focus->flags & VISIBLE))
+	      {
+		RaiseWindow (Scr.Focus);
+	      }
+	  }
+	return 0;
+      }
+#ifndef TIME_WITH_SYS_TIME
+    value.it_value.tv_usec = 0;
+    value.it_value.tv_sec = 0;
+#else
+    getitimer (ITIMER_REAL, &value);
+#endif
+    if (value.it_value.tv_sec > 0 || value.it_value.tv_usec > 0)
+      if (t == NULL || value.it_value.tv_sec < tv.tv_sec ||
+	  (value.it_value.tv_sec == tv.tv_sec && value.it_value.tv_usec < tv.tv_usec))
+	t = &value.it_value;
+  }
+#endif /* 1 */
 
   /* Do this IMMEDIATELY prior to select, to prevent any nasty
    * queued up X events from just hanging around waiting to be
    * flushed */
-  XFlush (dpy);
   if (XPending (dpy))
     {
-      XNextEvent (dpy, event);
-      StashEventTime (event);
-      return 1;
+	  XNextEvent (dpy, event);
+	  StashEventTime (event);
+	  return 1;
     }
   /* Zap all those zombies! */
   /* If we get to here, then there are no X events waiting to be processed.
    * Just take a moment to check for dead children. */
   ReapChildren ();
-
   XFlush (dpy);
-  if ((value.it_value.tv_usec != 0) ||
-      (value.it_value.tv_sec != 0))
+
 #ifdef __hpux
-    retval = select (fd_width, (int *) &in_fdset, 0, 0, &value.it_value);
+  retval = select (min(max_fd+1,fd_width), (int *) &in_fdset, (int *) &out_fdset, NULL, t);
 #else
-    retval = select (fd_width, &in_fdset, 0, 0, &value.it_value);
-#endif
-  else
-#ifdef __hpux
-    retval = select (fd_width, (int *) &in_fdset, (int *) &out_fdset,
-		     0, NULL);
-#else
-    retval = select (fd_width, &in_fdset, &out_fdset, 0, NULL);
+  retval = select (min(max_fd+1,fd_width), &in_fdset, &out_fdset, NULL, t);
 #endif
 
-  /* Check for module input. */
-  for (i = 0; i < npipes; i++)
+  /* check for incoming module connections */
+  if (module_fd >= 0 && FD_ISSET (module_fd, &in_fdset))
+  {
+    if (module_accept (module_fd) != -1)
+      fprintf (stderr, "accepted module connection\n");
+  }
+  if (retval > 0)
     {
-      if (readPipes[i] >= 0)
+      /* Check for module input. */
+      for (i = 0; i < npipes; i++)
 	{
-	  if ((retval > 0) && (FD_ISSET (readPipes[i], &in_fdset)))
-	    {
-	      if ((count =
-		   read (readPipes[i], &targetWindow, sizeof (Window))) > 0)
-		{
-		  HandleModuleInput (targetWindow, i);
-		}
-	      if (count <= 0)
-		{
-		  KillModule (i, 100);
-		}
-	    }
-	}
-      if (writePipes[i] >= 0)
-	{
-	  if ((retval > 0) && (FD_ISSET (writePipes[i], &out_fdset)))
-	    {
-	      FlushQueue (i);
-	    }
+	  if (Module[i].fd >= 0 && FD_ISSET (Module[i].fd, &in_fdset))
+	    HandleModuleInput (i);
+	  if (Module[i].fd >= 0 && FD_ISSET (Module[i].fd, &out_fdset))
+	    FlushQueue (i);
 	}
     }
+
+  /* handle timeout events */
+  timer_handle ();
   return 0;
 }
-
-	/* Change the icon, bugs for that to wiz@iol.it */
-void
-ChangeIcon ()
-{
-  char *cp;
-  if (!(Tmp_win->flags & SUPPRESSICON) && (Tmp_win->flags & ICON_OURS))
-    {
-      SearchIcon (Tmp_win, &cp);
-      if (cp && !(Tmp_win->icon_bitmap_file && cp == Tmp_win->icon_bitmap_file))
-	Tmp_win->icon_bitmap_file = cp;
-      else
-	return;
-    }
-  GetIcon (Tmp_win);
-  DrawIconWindow (Tmp_win);
-}
-
-void
-SearchIcon (ASWindow * tmp_win, char **value)
-{
-  name_list *nptr = Scr.TheList;
-  *value = tmp_win->icon_bitmap_file;
-
-  /* Start from the last element, because we are looking for
-     an icon name and it is the last match that should win,
-     I think better to start from the end of the list...;)
-   */
-
-  if (!nptr)
-    return;
-  nptr = nptr->prev;
-  do
-    {
-      if (nptr->value)
-	{
-	  if (matchWildcards (nptr->name, tmp_win->icon_name) == TRUE)
-	    {
-	      *value = nptr->value;
-	      return;
-	    }
-	  if (matchWildcards (nptr->name, tmp_win->name) == TRUE)
-	    {
-	      *value = nptr->value;
-	      return;
-	    }
-	  if (matchWildcards (nptr->name, tmp_win->class.res_name) == TRUE)
-	    {
-	      *value = nptr->value;
-	      return;
-	    }
-	  if (matchWildcards (nptr->name, tmp_win->class.res_class) == TRUE)
-	    {
-	      *value = nptr->value;
-	      return;
-	    }
-	}
-      nptr = nptr->prev;
-    }
-  while (nptr != Scr.TheList);
-}
