@@ -34,14 +34,21 @@
 #include <sys/time.h>
 #include <unistd.h>
 #include <signal.h>
+
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
 #include <X11/Xatom.h>
 
+#include "../../include/aftersteplib.h"
 #include "../../include/afterstep.h"
-#include "../../include/menus.h"
-#include "../../include/misc.h"
 #include "../../include/parse.h"
+#include "../../include/misc.h"
+#include "../../include/style.h"
 #include "../../include/screen.h"
 #include "../../include/module.h"
+#include "../../include/loadimg.h"
+
+#include "menus.h"
 
 XGCValues Globalgcv;
 unsigned long Globalgcm;
@@ -118,8 +125,7 @@ Destroy (ASWindow * Tmp_win, Bool kill_client)
   if (Tmp_win == Scr.Hilite)
     Scr.Hilite = NULL;
 
-  Broadcast (M_DESTROY_WINDOW, 3, Tmp_win->w, Tmp_win->frame,
-	     (unsigned long) Tmp_win, 0, 0, 0, 0);
+  Broadcast (M_DESTROY_WINDOW, 3, Tmp_win->w, Tmp_win->frame, (unsigned long) Tmp_win);
 
   if (Scr.PreviousFocus == Tmp_win)
     Scr.PreviousFocus = NULL;
@@ -165,18 +171,15 @@ Destroy (ASWindow * Tmp_win, Bool kill_client)
 
   XDeleteContext (dpy, Tmp_win->w, ASContext);
 
-/* 
-   if ((Tmp_win->icon_w)&&(Tmp_win->flags & PIXMAP_OURS))
-   XFreePixmap(dpy, Tmp_win->iconPixmap);
- */
+  if (Tmp_win->icon_pm_pixmap != None && Tmp_win->wmhints &&
+      !(Tmp_win->wmhints->flags & IconPixmapHint))
+    UnloadImage (Tmp_win->icon_pm_pixmap);
 
-#ifdef NO_ICON_BACKGROUND
-  if (Tmp_win->icon_w)
+  if (Tmp_win->icon_title_w)
     {
-      XDestroyWindow (dpy, Tmp_win->icon_w);
-      XDeleteContext (dpy, Tmp_win->icon_w, ASContext);
+      XDestroyWindow (dpy, Tmp_win->icon_title_w);
+      XDeleteContext (dpy, Tmp_win->icon_title_w, ASContext);
     }
-#endif
 
   if ((Tmp_win->flags & ICON_OURS) && (Tmp_win->icon_pixmap_w != None))
     XDestroyWindow (dpy, Tmp_win->icon_pixmap_w);
@@ -198,6 +201,13 @@ Destroy (ASWindow * Tmp_win, Bool kill_client)
       for (i = 0; i < 2; i++)
 	XDeleteContext (dpy, Tmp_win->corners[i], ASContext);
     }
+#ifndef NO_TEXTURE
+  if (Tmp_win->flags & FRAME)
+    {
+      frame_free_data (Tmp_win, False);
+    }
+#endif /* !NO_TEXTURE */
+
   Tmp_win->prev->next = Tmp_win->next;
   if (Tmp_win->next != NULL)
     Tmp_win->next->prev = Tmp_win->prev;
@@ -262,22 +272,19 @@ flush_expose (Window w)
 void
 RestoreWithdrawnLocation (ASWindow * tmp, Bool restart)
 {
-  int a, b, w2, h2;
-  unsigned int bw, mask;
+  int w2, h2;
+  unsigned int mask;
 
   XWindowChanges xwc;
 
   if (!tmp)
     return;
 
-  if (XGetGeometry (dpy, tmp->w, &JunkRoot, &xwc.x, &xwc.y,
-		    &JunkWidth, &JunkHeight, &bw, &JunkDepth))
+  if (XGetGeometry (dpy, tmp->w, &JunkRoot, &JunkX, &JunkY,
+		    &JunkWidth, &JunkHeight, &JunkBW, &JunkDepth))
     {
+      get_client_geometry (tmp, tmp->frame_x, tmp->frame_y, tmp->frame_width, tmp->frame_height, &xwc.x, &xwc.y, NULL, NULL);
 
-      XTranslateCoordinates (dpy, tmp->frame, Scr.Root, xwc.x, xwc.y,
-			     &a, &b, &JunkChild);
-      xwc.x = a + tmp->xdiff;
-      xwc.y = b + tmp->ydiff;
       xwc.border_width = tmp->old_bw;
       mask = (CWX | CWY | CWBorderWidth);
 
@@ -349,12 +356,10 @@ RestoreWithdrawnLocation (ASWindow * tmp, Bool restart)
 
       if ((tmp->flags & ICONIFIED) && (!(tmp->flags & SUPPRESSICON)))
 	{
-#ifdef NO_ICON_BACKGROUND
-	  if (tmp->icon_w)
-	    XUnmapWindow (dpy, tmp->icon_w);
-#endif
-	  if (tmp->icon_pixmap_w)
+	  if (tmp->icon_pixmap_w != None)
 	    XUnmapWindow (dpy, tmp->icon_pixmap_w);
+	  if (tmp->icon_title_w != None)
+	    XUnmapWindow (dpy, tmp->icon_title_w);
 	}
       XConfigureWindow (dpy, tmp->w, mask, &xwc);
 
@@ -363,14 +368,37 @@ RestoreWithdrawnLocation (ASWindow * tmp, Bool restart)
 }
 
 
+#if 0				/* (see SetTimer) */
+/**************************************************************************
+ * 
+ * For auto-raising windows, this routine is called
+ *
+ *************************************************************************/
+
+static void
+autoraise_timer_handler (void *data)
+{
+  Window child;
+  XQueryPointer (dpy, Scr.Root, &JunkRoot, &child, &JunkX, &JunkY, &JunkX,
+		 &JunkY, &JunkMask);
+  if (Scr.Focus != NULL && child == Scr.Focus->frame && !(Scr.Focus->flags & VISIBLE))
+    RaiseWindow (Scr.Focus);
+}
+#endif /* 0 */
+
 /***************************************************************************
  *
  * Start/Stops the auto-raise timer
  *
  ****************************************************************************/
+
 void
 SetTimer (int delay)
 {
+#if 1
+  /* unfortunately, a bug in glibc-2.0.7 causes the timer version of this 
+   * code to segfault; until glibc-2.0.7 is no longer in popular use, we 
+   * use the old code */
 #ifdef TIME_WITH_SYS_TIME
   struct itimerval value;
 
@@ -380,6 +408,11 @@ SetTimer (int delay)
   value.it_interval.tv_sec = 0;
   setitimer (ITIMER_REAL, &value, NULL);
 #endif
+#else /* 1 */
+  while (timer_remove_by_data (&autoraise_timer_handler));
+  if (delay > 0)
+    timer_new (delay, &autoraise_timer_handler, &autoraise_timer_handler);
+#endif /* 1 */
 }
 
 /***************************************************************************
@@ -546,7 +579,7 @@ MyXGrabKey (Display * display, int keycode, unsigned modifiers,
     {
       mod = *mods++;
       XGrabKey (display, keycode, modifiers | mod, grab_window, owner_events,
-		pointer_mode, keyboard_mode);
+		pointer_mode, keyboard_mode); // A memory leak occurs here. Bug in Xlib?
     }
   while (mod);
 }
@@ -594,54 +627,49 @@ void
 UpdateVisibility (void)
 {
   ASWindow *t, *s, *tbase;
-  int tx1, ty1, tx2, ty2;
-  int ontop = 1, onback = 1, visible;
 
   tbase = Scr.ASRoot.next;
   for (t = Scr.ASRoot.next; t != NULL; t = t->next)
     {
+      int visible = 0;
+      int tx1, ty1, tx2, ty2;
+
       if (t->flags & MAPPED)
 	{
 	  tx1 = t->frame_x;
 	  ty1 = t->frame_y;
-	  tx2 = t->frame_x + t->frame_width;
-	  ty2 = t->frame_y + t->frame_height;
+	  if (t->flags & SHADED)
+	    {
+	      tx2 = t->frame_x + t->title_width;
+	      ty2 = t->frame_y + t->title_height;
+	    }
+	  else
+	    {
+	      tx2 = t->frame_x + t->frame_width;
+	      ty2 = t->frame_y + t->frame_height;
+	    }
 	}
       else if (t->flags & ICONIFIED)
 	{
-	  tx1 = t->icon_x_loc;
-	  ty1 = t->icon_y_loc;
-	  tx2 = t->icon_x_loc + t->icon_p_width;
-	  ty2 = t->icon_y_loc + t->icon_p_height;
-	}
-      else if (t->flags & SHADED)
-	{
-	  tx1 = t->frame_x;
-	  ty1 = t->frame_y;
-	  tx2 = t->frame_x + t->frame_width;
-	  ty2 = t->frame_y + Scr.TitleHeight;
+	  tx1 = t->icon_p_x;
+	  ty1 = t->icon_p_y;
+	  tx2 = t->icon_p_x + t->icon_p_width;
+	  ty2 = t->icon_p_y + t->icon_p_height;
 	}
       else
 	continue;
 
-      if (ontop && !(t->flags & ONTOP))
-	{
-	  ontop = 0;
-	  tbase = t;
-	}
-      if (onback && !(t->flags & ONBACK))
-	{
-	  onback = 0;
-	  tbase = t;
-	}
       if ((tx2 > 0) && (tx1 < Scr.MyDisplayWidth) &&
 	  (ty2 > 0) && (ty1 < Scr.MyDisplayHeight))
 	{
 	  visible = VISIBLE;
-	  for (s = tbase; s != t; s = s->next)
+	  for (s = Scr.ASRoot.next; s != t; s = s->next)
 	    {
 	      if ((s->flags & TRANSIENT) && (s->transientfor == t->w))
 		continue;
+	      else if (s->layer != t->layer)
+		continue;
+
 	      if (s->flags & MAPPED)
 		{
 		  if ((tx2 > s->frame_x) && (tx1 < s->frame_x + s->frame_width) &&
@@ -653,8 +681,8 @@ UpdateVisibility (void)
 		}
 	      else if (s->flags & ICONIFIED)
 		{
-		  if ((tx2 > s->icon_x_loc) && (tx1 < s->icon_x_loc + s->icon_p_width) &&
-		      (ty2 > s->icon_y_loc) && (ty1 < s->icon_y_loc + s->icon_p_height))
+		  if ((tx2 > s->icon_p_x) && (tx1 < s->icon_p_x + s->icon_p_width) &&
+		      (ty2 > s->icon_p_y) && (ty1 < s->icon_p_y + s->icon_p_height))
 		    {
 		      visible = 0;
 		      break;
@@ -662,8 +690,8 @@ UpdateVisibility (void)
 		}
 	      else if (s->flags & SHADED)
 		{
-		  if ((tx2 > s->frame_x) && (tx1 < s->frame_x + s->frame_width) &&
-		  (ty2 > s->frame_y) && (ty1 < s->frame_y + Scr.TitleHeight))
+		  if ((tx2 > s->frame_x) && (tx1 < s->frame_x + s->title_width) &&
+		  (ty2 > s->frame_y) && (ty1 < s->frame_y + s->title_height))
 		    {
 		      visible = 0;
 		      break;
@@ -671,8 +699,6 @@ UpdateVisibility (void)
 		}
 	    }
 	}
-      else
-	visible = 0;
       if ((t->flags & VISIBLE) != visible)
 	{
 	  t->flags ^= VISIBLE;
@@ -688,109 +714,68 @@ UpdateVisibility (void)
     }
 }
 
-/******************************************************************************
- *
- * Get the correct window stacking order from the X server and
- * make sure everything that depends on the order is fine and dandy
- * 
- *****************************************************************************/
-
-void
-CorrectStackOrder (void)
+char *
+fit_vertical_text (MyFont font, char *text, int len, int maxheight)
 {
-  Window root, parent, *children, *cp;
-  unsigned nchildren;
-  ASWindow *t;
+  int savelen, i, h;
+  char *trunct;
 
-  if (XQueryTree (dpy, Scr.ASRoot.w, &root, &parent, &children,
-		  &nchildren))
-    {
-      for (cp = children; nchildren-- > 0; cp++)
-	{
-	  if ((XFindContext (dpy, *cp, ASContext, (caddr_t *) & t) == XCSUCCESS)
-	      && (t->frame == *cp))
-	    {
-	      if (t != Scr.ASRoot.next)
-		{
-		  t->prev->next = t->next;
-		  if (t->next)
-		    t->next->prev = t->prev;
-		  t->next = Scr.ASRoot.next;
-		  t->prev = &Scr.ASRoot;
-		  Scr.ASRoot.next->prev = t;
-		  Scr.ASRoot.next = t;
-		}
-	    }
-	}
-      XFree (children);
-    }
-  else
-    {
-      fprintf (stderr, "CorrectStackOrder(): XQueryTree failed!\n");
-    }
+  savelen = len;
+  h = len * font.height;
 
-  t = Scr.ASRoot.next;
-  if (t)
+  while (h > maxheight && len > 0)
     {
-      if (t->flags & ONTOP)
+      len -= 1;
+      h = len * font.height;
+    }
+  if (len <= 0)
+    len = 0;
+  trunct = (char *) safemalloc (len * sizeof (char) + 1);
+  memset (trunct, '\0', len * sizeof (char) + 1);
+  strncpy (trunct, text, len);
+
+  if (savelen != len)
+    {
+      for (i = 1; i <= 3; i++)
 	{
-	  do
-	    {
-	      t = t->next;
-	    }
-	  while (t && (t->flags & ONTOP));
-	  if (t)
-	    {
-	      do
-		{
-		  t = t->next;
-		}
-	      while (t && !(t->flags & ONTOP));
-	      if (t)
-		RaiseWindow (Scr.ASRoot.next);
-	    }
-	}
-      else
-	{
-	  do
-	    {
-	      t = t->next;
-	    }
-	  while (t && !(t->flags & ONTOP));
-	  if (t)
-	    RaiseWindow (t);
+	  len -= 1;
+	  if (len <= 0)
+	    break;
+	  trunct[len] = '.';
 	}
     }
-  if (t)
+  return trunct;
+}
+
+char *
+fit_horizontal_text (MyFont font, char *text, int len, int maxwidth)
+{
+  int w, savelen, i;
+  char *trunct = NULL;
+
+  savelen = len;
+  w = XTextWidth (font.font, text, len);
+  while (w > maxwidth && len > 0)
     {
-      if (t->flags & ONBACK)
-	{
-	  do
-	    {
-	      t = t->next;
-	    }
-	  while (t && (t->flags & ONBACK));
-	  if (t)
-	    {
-	      do
-		{
-		  t = t->next;
-		}
-	      while (t && !(t->flags & ONBACK));
-	      if (t)
-		LowerWindow (Scr.ASRoot.next);
-	    }
-	}
-      else
+      len -= 1;
+      w = XTextWidth (font.font, text, len);
+    }
+  if (len <= 0)
+    len = 0;
+  trunct = (char *) safemalloc (len * sizeof (char) + 1);
+  memset (trunct, '\0', len * sizeof (char) + 1);
+  strncpy (trunct, text, len);
+
+  if (savelen != len)
+    {
+      for (i = 1; i <= 3; i++)
 	{
-	  do
-	    {
-	      t = t->next;
-	    }
-	  while (t && !(t->flags & ONBACK));
-	  if (t)
-	    LowerWindow (t);
+	  len -= 1;
+	  if (len <= 0)
+	    break;
+	  trunct[len] = '.';
 	}
     }
-  UpdateVisibility ();
+
+  return trunct;
 }
